/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "MountDialog.h"
#include "QDebug.h"
#include "DialogHelper.h"
#include "VolumeConfig.h"
#include "Pline/MountUtil.h"

#include <glibmm/i18n.h>
#include <assert.h>

//==============================================================================

MountDialog::MountDialog(   BaseObjectType* cobject,
							const Glib::RefPtr<Gtk::Builder>& refGlade)
: QDialog(cobject)
{
	set_modal(true);
	set_title(_("Mount Volume"));
	
	gpoDialogHelper = 0;
	gpoVolumeConfig = 0;
	gpvecVolumeConfigLabel = 0;
	gpoSourceFileChooser = 0;

	gbAddOnly = false;
	
	UtilGui::mapMountDlg(refGlade, gtMountDlg);	
	
	gtMountDlg.poAddButton->signal_clicked().connect(
		sigc::mem_fun(*this, &MountDialog::onAdd));

	gtMountDlg.poCancelButton->signal_clicked().connect(
		sigc::mem_fun(*this, &MountDialog::onCancel));
	
	gtMountDlg.poMountButton->signal_clicked().connect(
		sigc::mem_fun(*this, &MountDialog::onMount));
		
	gtMountDlg.poPermToggleButton->signal_clicked().connect(
		sigc::mem_fun(*this, &MountDialog::onPermissions));
		
	gtMountDlg.poReadOnlyCheckButton->signal_toggled().connect(
		sigc::mem_fun(*this, &MountDialog::onReadOnly));
		
	gtMountDlg.poEncryptedCheckButton->signal_toggled().connect(
		sigc::mem_fun(*this, &MountDialog::onEncrypt));
	
	gtMountDlg.poUsePasswordRadioButton->signal_toggled().connect(
		sigc::mem_fun(*this, &MountDialog::onUsePassword));
	
	gtMountDlg.poUseKeyFileRadioButton->signal_toggled().connect(
		sigc::mem_fun(*this, &MountDialog::onUseKeyFile));

	Gtk::ComboBoxEntryText* pocmbFSys = manage(new Gtk::ComboBoxEntryText);
			
	gtMountDlg.poFSysVBox->pack_start(*pocmbFSys, false, false);
	gtMountDlg.poFSysComboBox = pocmbFSys;
	
	Gtk::ComboBoxText* pocmbPlaceAfter = manage(new Gtk::ComboBoxText);
			
	gtMountDlg.poPlaceAfterVBox->pack_start(*pocmbPlaceAfter, false, false);
	gtMountDlg.poPlaceAfterComboBox = pocmbPlaceAfter;
	
	show_all_children();
}

//==============================================================================

MountDialog::~MountDialog()
{
	if (gpoDialogHelper)
		delete gpoDialogHelper;
	
	if (gpoSourceFileChooser)
		delete gpoSourceFileChooser;
}

//==============================================================================

void MountDialog::buildFSys()
{
	unsigned int xui;

	gvecFSys.clear();
	
	gtMountDlg.poFSysComboBox->clear_items();
	gtMountDlg.poFSysComboBox->append_text(_("-- Auto"));
	
	Pline::MountUtil::getFSysList(&gvecFSys);

	for (xui=0; xui<gvecFSys.size(); ++xui)
		gtMountDlg.poFSysComboBox->append_text(gvecFSys[xui]);
}

//==============================================================================

void MountDialog::onAdd()
{
	gbAddOnly = true;
	
	updateData(false);
	
	if (!validate())
		return;
		
	QDialog::response(Gtk::RESPONSE_OK);
}

//==============================================================================

void MountDialog::onCancel()
{
	QDialog::response(Gtk::RESPONSE_CANCEL);
}

//==============================================================================

void MountDialog::onEncrypt()
{
	gpoDialogHelper->onEncrypt();
}

//==============================================================================

void MountDialog::onMount()
{
	updateData(false);
	
	if (!validate())
		return;
		
	QDialog::response(Gtk::RESPONSE_OK);
}

//==============================================================================

void MountDialog::onPermissions()
{
	gpoDialogHelper->onPermissions();
}

//==============================================================================

void MountDialog::onReadOnly()
{
	gpoDialogHelper->onFSysReadOnly();
}

//==============================================================================

void MountDialog::onSource()
{
	gpoDialogHelper->onSource();
}

//==============================================================================

void MountDialog::onUseKeyFile()
{
	gpoDialogHelper->onUseKeyFile();
}

//==============================================================================

void MountDialog::onUsePassword()
{
	gpoDialogHelper->onUsePassword();
}

//==============================================================================

void MountDialog::setData(  VolumeConfig *poVolumeConfig,
							std::vector<Glib::ustring> *pvecVolumeConfigLabel,
							const Glib::ustring& stSrc)
{
	gpoVolumeConfig = poVolumeConfig;
	gpvecVolumeConfigLabel = pvecVolumeConfigLabel;
	
	buildFSys();
	
	gbAddOnly = false;
	
	// work around for bug in FileChooserButton (on cancel the control would be messed up)
	if (gpoSourceFileChooser)
		delete gpoSourceFileChooser;
	
	gpoSourceFileChooser = new Gtk::FileChooserButton;
	
	gtMountDlg.poSourceVBox->pack_start(*gpoSourceFileChooser, false, false);
	gtMountDlg.poSourceFileChooser = gpoSourceFileChooser;
	
	gtMountDlg.poSourceFileChooser->signal_selection_changed().connect(
		sigc::mem_fun(*this, &MountDialog::onSource));
	
	show_all_children();
	
	//
	// Dialog Helper
	//
	
	if (gpoDialogHelper)
		delete gpoDialogHelper;

	gpoDialogHelper = new DialogHelper(this, gpoVolumeConfig);
	
	// mount
	gpoDialogHelper->setFSysComboBox(gtMountDlg.poFSysComboBox);
	gpoDialogHelper->setFSysList(&gvecFSys);
	gpoDialogHelper->setMountPointFileChooser(gtMountDlg.poMountPointFileChooser);
	gpoDialogHelper->setPermToggleButton(gtMountDlg.poPermToggleButton);
	gpoDialogHelper->setPlaceAfterComboBox(gtMountDlg.poPlaceAfterComboBox);
	gpoDialogHelper->setPlaceAfterList(gpvecVolumeConfigLabel);
	gpoDialogHelper->setReadOnlyCheckButton(gtMountDlg.poReadOnlyCheckButton);
	gpoDialogHelper->setSourceFileChooser(gtMountDlg.poSourceFileChooser);
	gpoDialogHelper->setVolumeLabelEntry(gtMountDlg.poVolumeLabelEntry);
		
	gpoDialogHelper->setEncryptedCheckButton(gtMountDlg.poEncryptedCheckButton);
	gpoDialogHelper->setKeyFileFileChooser(gtMountDlg.poKeyFileFileChooser);
	gpoDialogHelper->setUseKeyFileRadioButton(gtMountDlg.poUseKeyFileRadioButton);
	gpoDialogHelper->setUsePasswordRadioButton(gtMountDlg.poUsePasswordRadioButton);
	
	gpoDialogHelper->buildPlaceAfter();
		
	updateData(true);
	
	if (!stSrc.empty())
		gtMountDlg.poSourceFileChooser->set_filename(stSrc);
}

//==============================================================================

void MountDialog::updateData(const bool bToScreen)
{
	assert(gpoVolumeConfig);
	
	gpoDialogHelper->updateMountData(bToScreen);
	gpoDialogHelper->updateCryptData(bToScreen);
}

//==============================================================================

bool MountDialog::validate()
{
	if (!gpoDialogHelper->validateMountData(true))
		return false;
	
	if (gpoVolumeConfig->getCryptData())
	{
		if (!gpoDialogHelper->validateCryptData())
			return false;
	}
	
	return true;
}

//==============================================================================
