/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "ClonePline.h"
#include "QDebug.h"
#include "../Util.h"

#include "CloneData.h"
#include "CopyFilter.h"

#include <iostream>
#include <iomanip>
#include <glibmm/i18n.h>

namespace Pline
{

//==============================================================================

ClonePline::ClonePline( const eDataMode eDataModeItem,
						VolumeConfig *poVolumeConfig)
: ProcessBasePline(eDataModeItem, poVolumeConfig)
{
	gfCurrentSpeed = 0;
	giPctDone = 0;
	
	CopyFilter *poCopyFilter = new CopyFilter;
			
	// set data from config
	CloneData* poCloneData = goVolumeConfig.getCloneData();
	
	if (!poCloneData)
	{
		poCloneData = new CloneData(goVolumeConfig.getData());
		goVolumeConfig.setCloneData(poCloneData);
	}
	
	// save initial status	
	gbCanOverwriteInit = poCloneData->getCanOverwrite();
	
	// set filter data
	poCopyFilter->setSrcData(poCloneData);
	// set filter run mode
	poCopyFilter->setRunMode(CopyFilter::kRunCopyFileStartNew);
	// set other data
	poCopyFilter->setCurrentSpeed(&gfCurrentSpeed);
	poCopyFilter->setETA(&gstETA);
	poCopyFilter->setPctDone(&giPctDone);
	
	
	// add filter to pline
	addFilter(poCopyFilter);
}

//==============================================================================

ClonePline::~ClonePline()
{
}

//==============================================================================

const Glib::ustring ClonePline::getInfoMessage()
{
	Glib::ustring stMsg;
	
	Glib::ustring stSpeed = 
		Glib::ustring::format(std::fixed, std::setprecision(2), gfCurrentSpeed);
			
	stMsg = Util::compose(	_("Cloning at %sM/s, %d%% done. ETA %s..."),
							stSpeed.c_str(),
							giPctDone,
							(gstETA.empty()) ? Glib::ustring(_("inf")).c_str() : gstETA.c_str());
	
	return stMsg;
}

//==============================================================================

bool ClonePline::onFilterRun(Filter *poFilter, bool bHasRunWithErrors)
{
	// set inital overwrite flag
	if (poFilter->getFilterName() == "Copy")
	{
		if (goVolumeConfig.getCloneData())
			goVolumeConfig.getCloneData()->setCanOverwrite(gbCanOverwriteInit);
	}
	
	return Pline::onFilterRun(poFilter, bHasRunWithErrors);
}

//==============================================================================

} // Pline	
