/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "MkCryptFilter.h"
#include "../QDebug.h"
#include "../eMountSys.h"
#include "../Util.h"
#include "../VolumeDialog.h"
#include "CryptData.h"
#include "CryptUtil.h"


#include <glibmm/i18n.h>
#include <assert.h>

namespace Pline
{

//==============================================================================

MkCryptFilter::MkCryptFilter()
: Filter()
{
	gpoCryptData = 0;
	
	gciPasswdTry = 0;
}

//==============================================================================

MkCryptFilter::~MkCryptFilter()
{
}

//==============================================================================

void MkCryptFilter::askData()
{
	if (eMountSys::getInstance().getRunMode() != eMountSys::kRunGui)
		return;
		
	if (!isValidPassword())
	{
		if (gciPasswdTry < 2)
			gciPasswdTry++;
		
		// first
		VolumeDialog::tConfig tConfigItem;
		VolumeDialog::initConfig(tConfigItem);
			
		tConfigItem.bAutoRun = false;
		tConfigItem.bWaitData = false;
		tConfigItem.bAskPassword = true;
		tConfigItem.stLabel = gpoCryptData->getLabel();
		tConfigItem.stSrc = gpoCryptData->getSrc();
		tConfigItem.stCommand = getFilterName();
		
		if (gciPasswdTry > 1)
			tConfigItem.stMsg = _("Password did not match. Try again.");
		else
			tConfigItem.stMsg = _("Enter Password to Secure Volume.");
		
		VolumeDialog oDlg(tConfigItem);
			
		if (oDlg.run() != Gtk::RESPONSE_OK)
		{
			oDlg.hide();
			
			while (Gtk::Main::events_pending()) // force redraw
				Gtk::Main::iteration();
				
			cancelDataReq();
			return;
		}

		gstPasswd1 = oDlg.getPassword();
			
		oDlg.hide();
		
		// second
		VolumeDialog::tConfig tConfigItem2;
		VolumeDialog::initConfig(tConfigItem2);
			
		tConfigItem2.bAutoRun = false;
		tConfigItem2.bWaitData = false;
		tConfigItem2.bAskPassword = true;
		tConfigItem2.stLabel = gpoCryptData->getLabel();
		tConfigItem2.stSrc = gpoCryptData->getSrc();
		tConfigItem2.stCommand = getFilterName();
		tConfigItem2.stMsg = _("Validate password.");
		
		VolumeDialog oDlg2(tConfigItem2);
			
		if (oDlg2.run() != Gtk::RESPONSE_OK)
		{
			oDlg2.hide();
			
			while (Gtk::Main::events_pending()) // force redraw
				Gtk::Main::iteration();
				
			cancelDataReq();
			return;
		}

		gstPasswd2 = oDlg2.getPassword();
			
		oDlg.hide();
			
		while (Gtk::Main::events_pending()) // force redraw
			Gtk::Main::iteration();
	}
	
	Filter::askData();
}

//==============================================================================

const bool MkCryptFilter::isValidPassword()
{
	return ((gstPasswd1 == gstPasswd2 && !gstPasswd1.empty())
			|| !gpoCryptData->getKeyFile().empty());
}

//==============================================================================

bool MkCryptFilter::run()
{
	assert(gpoCryptData);
	
	// reset error
	gstLastError.clear();
	
	// ask password
	while (true)
	{
		if (!isValidPassword())
		{
			requestData();	

			if (!waitData())
				return false;
		}
		else
			break;
	}
	
	// format
	int ci = 0;

	for (ci=0; ci<MKCRYPTFILTER_MAXTRY; ++ci)
	{
		if (CryptUtil::formatCrypt(	gpoCryptData->getSrc(),
									gstPasswd1,
									gpoCryptData->getKeyFile(),
									gstCipher,
									gstCBC,
									gstKeySize,
									&gstLastError))
		{
			gstPasswd1.clear();
			gstPasswd2.clear();
			
			return true;
		}
		
		Glib::usleep(MKCRYPTFILTER_SLEEP_TRY);
	}
	
	gstPasswd1.clear();
	gstPasswd2.clear();
	
	return false;
}

//==============================================================================

void MkCryptFilter::setSrcData(CryptData *poData)
{
	gpoCryptData = poData;
	
	Filter::setSrcData(poData);
}

//==============================================================================

} // Pline
