/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "ProgressDialog.h"
#include "QDebug.h"

//==============================================================================

ProgressDialog::ProgressDialog(const tConfig& tConfigItem)
: QDialog(getQDialogConfig(tConfigItem))
{
	gbCancel = false;

	// vbox
	Gtk::VBox* vbox = QDialog::getVBox();
	
	gpoMsgLabel = manage(new Gtk::Label(tConfigItem.stMsg, 0.0, 0.0));
	gpoMsgLabel->set_alignment(0.5);
	gpoMsgLabel->set_use_markup(true);
	gpoMsgLabel->set_line_wrap(true);

	vbox->pack_start(*gpoMsgLabel, false, false, 12);
	
	gpoProgressBar = manage(new Gtk::ProgressBar);
	
	vbox->pack_start(*gpoProgressBar, false, false, 12);
	
	// ok button
	Gtk::Button* button = add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	button->signal_clicked().connect(
		sigc::mem_fun(*this, &ProgressDialog::onCancel));
	
	show_all();
	
	if (tConfigItem.bAutoShow)
	{
		show_all_children();
		
		while (Gtk::Main::events_pending()) // force redraw
			Gtk::Main::iteration();
	}
}

//==============================================================================

ProgressDialog::~ProgressDialog()
{
}

//==============================================================================

QDialog::tConfig ProgressDialog::getQDialogConfig(const tConfig& tConfigItem)
{
	QDialog::tConfig tQDialogConfigItem;
	QDialog::initConfig(tQDialogConfigItem);
	
	tQDialogConfigItem.iWidth = 400;
	tQDialogConfigItem.stTitle = tConfigItem.stTitle;
	tQDialogConfigItem.bWaitData = tConfigItem.bWaitData;
	
	return tQDialogConfigItem;
}

//==============================================================================

void ProgressDialog::initConfig(tConfig& tConfigItem)
{
	tConfigItem.bAutoShow = true;
	tConfigItem.bWaitData = true;
	tConfigItem.stTitle.clear();
	tConfigItem.stMsg.clear();
}

//==============================================================================

void ProgressDialog::onCancel()
{
	gbCancel = true;
}

//==============================================================================

bool ProgressDialog::update(const double dFraction,
							const Glib::ustring& stMsg)
{
	gpoMsgLabel->set_label(stMsg);
	gpoProgressBar->set_fraction(dFraction);
	
	while (Gtk::Main::events_pending()) // force redraw
		Gtk::Main::iteration();
		
	return (gbCancel) ? false : true;
}

//==============================================================================
