/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "Util.h"
#include "QDebug.h"

#include <iostream>
#include <fstream>

#include <glib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>

#include <glibmm/i18n.h>
#include <glib.h>
#include <assert.h>

#define MODPROBE_PATH		"/sbin/modprobe"
#define USERSFILE_PATH		"/etc/passwd"
#define GROUPFILE_PATH		"/etc/group"

//==============================================================================

Util::Util()
{
}

//==============================================================================

Util::~Util()
{
}

//==============================================================================

bool Util::base64Dec(const Glib::ustring& st, Glib::ustring *pst64Dec)
{
	if (st.empty())
		return false;
		
	gsize iLen = 0;
	guchar* pstDec = g_base64_decode((const gchar*)st.c_str(), &iLen);
	
	if (pstDec)
	{
		*pst64Dec = Glib::ustring((const gchar*)pstDec);
		
		g_free(pstDec);
		
		return true;
	}
	
	return false;
}

//==============================================================================

bool Util::base64Enc(const Glib::ustring& st, Glib::ustring *pst64Enc)
{
	if (st.empty())
		return false;
		
	gchar* pstEnc = g_base64_encode((const guchar*)st.c_str(), st.length());
	
	if (pstEnc)
	{
		*pst64Enc = Glib::ustring(pstEnc);
		
		g_free(pstEnc);
		
		return true;
	}
	
	return false;
}
			
//==============================================================================

const Glib::ustring Util::compose(const char *format, ...)
{
	assert(format);
	
	va_list args;
	gchar *msg = NULL;

	va_start(args, format);
	msg = g_strdup_vprintf(format, args);
	va_end(args);
	
	Glib::ustring st = msg;
	
	g_free(msg);
	
	return st;
}
										
//==============================================================================

bool Util::exec(const Glib::ustring& stCmd,
				Glib::ustring *pstOutBuf)
{
	FILE *ptFile = NULL;
	int ich;
	Glib::ustring stCmdExec;
	
	if (pstOutBuf)
		pstOutBuf->clear();

	stCmdExec = Util::compose("%s 2>&1", stCmd.c_str());
	
	if (!(ptFile = popen(stCmdExec.c_str(), "r")))
		return false;

	if (pstOutBuf)
	{
		Glib::ustring stOutBuf;
		
		while ((ich = fgetc(ptFile)) != EOF)
		{
			stOutBuf.append(Util::compose("%c", (const char)ich));
			*pstOutBuf = stOutBuf;
		}
	}
	
	if (pclose(ptFile) == 0)
		return true;
		
	return false;
}

//==============================================================================

bool Util::file2string( const std::string& stFilename,
						Glib::ustring *pstData)
{
	/*std::ifstream oInFile(stFilename.c_str());
	Glib::ustring stData;
	Glib::ustring stStream;

	if (oInFile)
	{
		while (oInFile.good())	
			stData.append(Util::compose("%c", (const char)oInFile.get()));
		
		oInFile.close();
		
		*pstData = stData.substr(0, stData.size()-1);

		return true;
	}
	
	return false;*/
	try
	{
		*pstData = Glib::file_get_contents(stFilename);
	}
	catch (...)
	{
		return false;
	}
	
	return true;
}

//==============================================================================

bool Util::findSt(const Glib::ustring& st, std::vector<Glib::ustring>& vec)
{
	for (   std::vector<Glib::ustring>::size_type xi = 0;
			xi < vec.size();
			++xi)
	{
		if (vec[xi] == st)
			return true;
	}
	
	return false;
}

//==============================================================================

bool Util::getFileSize(	const std::string& stFilename,
						unsigned long long *pullSize)
{
	if (!isRegularFile(stFilename))
		return false;
	
	struct stat64 statbuf;
	
	if (stat64((const char*)stFilename.c_str(), &statbuf) != 0)
		return false;
	
	*pullSize = (unsigned long long)statbuf.st_size;
	
	return true;
}
							
//==============================================================================

bool Util::getGroups(   std::vector<Glib::ustring>* pvecGroupName,
						std::vector<int>* pvecGroupID,
						Glib::ustring *pstErrMsg)
{
	Glib::ustring stOutBuf;
	size_t iPos, iPosEndl;
	
	if (!file2string(GROUPFILE_PATH, &stOutBuf))
	{
		if (pstErrMsg)
			*pstErrMsg = Util::compose(_("Could not open %s."), GROUPFILE_PATH);
			
		return false;
	}

	while ((iPosEndl = stOutBuf.find("\n")) != std::string::npos)
	{
		Glib::ustring stLine = stOutBuf.substr(0, iPosEndl);
		int ci = 0;	
		
		while ((iPos = stLine.find(":")) != std::string::npos)	
		{
			Glib::ustring stField = stLine.substr(0, iPos);
			
			if (ci == 0)
				pvecGroupName->push_back(stField);
			else if (ci == 2)
				pvecGroupID->push_back(atoi(stField.c_str()));
				
			stLine = stLine.substr(iPos+1);
			ci++;
		}
		
		stOutBuf = stOutBuf.substr(iPosEndl+1);
	}
	
	if (pvecGroupName->size() != pvecGroupID->size())
	{
		if (pstErrMsg)
			*pstErrMsg = Util::compose(_("Could not read %s."), GROUPFILE_PATH);
	}
			
	return (pvecGroupName->size() == pvecGroupID->size());
}

//==============================================================================

bool Util::getUsers(std::vector<Glib::ustring>* pvecUserName,
					std::vector<int>* pvecUserID,
					Glib::ustring *pstErrMsg)
{
	Glib::ustring stOutBuf;
	size_t iPos, iPosEndl;
	
	if (!file2string(USERSFILE_PATH, &stOutBuf))
	{
		if (pstErrMsg)
			*pstErrMsg = Util::compose(_("Could not read %s."), USERSFILE_PATH);
			
		return false;
	}

	while ((iPosEndl = stOutBuf.find("\n")) != std::string::npos)
	{
		Glib::ustring stLine = stOutBuf.substr(0, iPosEndl);
		int ci = 0;	
		
		while ((iPos = stLine.find(":")) != std::string::npos)	
		{
			Glib::ustring stField = stLine.substr(0, iPos);
			
			if (ci == 0)
				pvecUserName->push_back(stField);
			else if (ci == 2)
				pvecUserID->push_back(atoi(stField.c_str()));
				
			stLine = stLine.substr(iPos+1);
			ci++;
		}
		
		stOutBuf = stOutBuf.substr(iPosEndl+1);
	}
	
	if (pvecUserName->size() != pvecUserID->size())
	{
		if (pstErrMsg)
			*pstErrMsg = Util::compose(_("Could not read %s."), USERSFILE_PATH);
	}
	
	return (pvecUserName->size() == pvecUserID->size());
}

//==============================================================================

bool Util::isDevice(const Glib::ustring& stDevice, Glib::ustring *pstErrMsg)
{
	struct stat statbuf;

	if (stDevice.empty())
	{
		if (pstErrMsg)
			*pstErrMsg = _("Invalid device.");
			
		return false;
	}
	
	if (stat((const char*)stDevice.c_str(), &statbuf) != 0)
	{
		if (pstErrMsg)
			*pstErrMsg = Util::compose(_("%s not found."), stDevice.c_str());

		return false;
	}
	
	if (!S_ISBLK(statbuf.st_mode))
	{
		if (pstErrMsg)
			*pstErrMsg = Util::compose(_("%s not a device."), stDevice.c_str());

		return false;
	}
	
	return true;
}

//==============================================================================

bool Util::isDirectory(const Glib::ustring& stDir, Glib::ustring *pstErrMsg)
{
	struct stat statbuf;

	if (stDir.empty())
	{
		if (pstErrMsg)
			*pstErrMsg = _("Invalid directory.");
			
		return false;
	}
	
	if (stat((const char*)stDir.c_str(), &statbuf) != 0)
	{
		if (pstErrMsg)
			*pstErrMsg = Util::compose(_("%s not found."), stDir.c_str());
			
		return false;
	}
	
	if (!S_ISDIR(statbuf.st_mode))
	{
		if (pstErrMsg)
			*pstErrMsg = Util::compose(_("%s not a directory."), stDir.c_str());
			
		return false;
	}
	
	return true;
}

//==============================================================================

bool Util::isFileExist( const Glib::ustring& stFilename, 
						Glib::ustring *pstErrMsg)
{
	struct stat statbuf;

	if (stFilename.empty())		
	{	
		if (pstErrMsg)
			*pstErrMsg = _("Invalid filename.");
		
		return false;
	}
	
	if (stat((const char*)stFilename.c_str(), &statbuf) != 0)
	{
		if (pstErrMsg)
			*pstErrMsg = Util::compose(_("%s not found."), stFilename.c_str());
			
		return false;
	}
	
	return true;
}

//==============================================================================

bool Util::isRegularFile(   const Glib::ustring& stFilename, 
							Glib::ustring *pstErrMsg)
{
	struct stat statbuf;

	if (stFilename.empty())
	{
		if (pstErrMsg)
			*pstErrMsg = _("Invalid filename.");
			
		return false;
	}
	
	if (stat((const char*)stFilename.c_str(), &statbuf) != 0)
	{
		if (pstErrMsg)
			*pstErrMsg = Util::compose(_("%s not found."), stFilename.c_str());

		return false;
	}
	
	if (!S_ISREG(statbuf.st_mode))
	{
		if (pstErrMsg)
			*pstErrMsg = Util::compose(_("%s not a regular file."), stFilename.c_str());

		return false;
	}
	
	return true;
}

//==============================================================================

bool Util::loadKernelModule(const Glib::ustring& stFilename)
{
	Glib::ustring stCmd;
	Glib::ustring stOut;
	
	if (!isFileExist(MODPROBE_PATH))
		return false;
	
	if (!isRegularFile(stFilename))
		return false;
	
	std::string stModName = Glib::path_get_basename(stFilename.c_str());

	if (stModName.size()>3 && stModName.substr(stModName.size() - 3) == ".ko")
		stModName.resize(stModName.size() - 3);
		
	stCmd = Util::compose("%s %s", MODPROBE_PATH, stModName.c_str());
	
	return exec(stCmd, &stOut);
}

//==============================================================================

bool Util::string2file( const Glib::ustring& stData,
						const std::string& stFilename)
{
	std::ofstream oOutFile(stFilename.c_str());

	if (oOutFile)
	{
		oOutFile << stData;
		oOutFile.close();

		return true;
	}
	
	return false;
}

//==============================================================================
