/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef VOLUMECONFIG_H
#define VOLUMECONFIG_H

#include <gtkmm.h>

#include "Pline/Data.h"
#include "Pline/CloneData.h"
#include "Pline/CryptData.h"
#include "Pline/MountData.h"

class VolumeConfig
{
public:
	VolumeConfig();
	~VolumeConfig();

	VolumeConfig* operator=(const VolumeConfig &oVolumeConfig);
	bool operator==(const VolumeConfig &oVolumeConfig) const;
	bool operator!=(const VolumeConfig &oVolumeConfig) const;
	
	// data
	inline Pline::Data& getData() {return *gpoData;}
	inline Pline::Data* getDataPtr() {return gpoData;}
	inline void setData(Pline::Data* po) {gpoData = po;}
	
	// clone data
	inline Pline::CloneData* getCloneData() {return gpoCloneData;}
	inline void removeCloneData() {delete gpoCloneData; gpoCloneData = 0;}
	inline void setCloneData(Pline::CloneData* po) {gpoCloneData = po;}

	// crypt data
	inline Pline::CryptData* getCryptData() {return gpoCryptData;}
	inline void removeCryptData() {delete gpoCryptData; gpoCryptData = 0;}
	inline void setCryptData(Pline::CryptData* po) {gpoCryptData = po;}
	
	// mount data
	inline Pline::MountData* getMountData() {return gpoMountData;}
	inline void setMountData(Pline::MountData* po) {gpoMountData = po;}
	
	// options
	inline bool isAutomount() {return gbAutomount;}
	inline void setAutomount(const bool b) {gbAutomount = b;}
	
	// config
	bool loadConfig(const Glib::ustring& stGroupName,
					Glib::KeyFile& oKeyFile);
	bool removeConfig(Glib::KeyFile& oKeyFile);
	bool saveConfig(Glib::KeyFile& oKeyFile);
		
	// compare
	static bool cmpByPos(VolumeConfig* po1, VolumeConfig* po2);

protected:

private:
	Glib::ustring gstGroupName;
	
	Pline::Data		 *gpoData;
	Pline::CloneData *gpoCloneData;
	Pline::CryptData *gpoCryptData;
	Pline::MountData *gpoMountData;
	
	bool gbAutomount;
	
	const Glib::ustring getGroupNameFromLabel();
};
	
#endif /* VOLUMECONFIG_H */
