/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "VolumeDialog.h"
#include "QDebug.h"
#include "Icon.h"
#include "QConfig.h"
#include "Util.h"
#include "PermDialog.h"
#include "eMountSys.h"
#include "eMountWin.h"

#include <assert.h>

#include <glibmm/i18n.h>

#define LINE_WRAP_LEN 100

//==============================================================================

VolumeDialog::VolumeDialog(const tConfig& tConfigItem)
: QDialog(getQDialogConfig(tConfigItem))
{
	bool bCancelButton = false;
	bool bFocusSet = false;
	
	gpoPasswdEntry = 0;
	gpoSaveAsEntry = 0;
	gpoSelectDirEntry = 0;
	gpoOverwriteCheckButton = 0;
	gpoMountData = 0;
	gpoPermToggleButton = 0;
	
	// set MountSrc and MountDir
	VolumeConfig oVolumeConfig;
	Glib::ustring stMountSrc = _("Source: n/a");
	Glib::ustring stMountDir =_("Mount Point: n/a");
	
	if (QConfig::getInstance().findVolume(tConfigItem.stLabel, oVolumeConfig)
		&& oVolumeConfig.getMountData())
	{
		if (!oVolumeConfig.getMountData()->getSrc().empty())
			stMountSrc = oVolumeConfig.getMountData()->getSrc();
		
		if (!oVolumeConfig.getMountData()->getDir().empty())
			stMountDir = oVolumeConfig.getMountData()->getDir();
	}
					
	// vbox
	Gtk::VBox* vbox = QDialog::getVBox();
	
	//
	// label with src and dest
	//
	Gtk::VBox* vbox4 = manage(new Gtk::VBox(false, 3));
	vbox->pack_start(*vbox4);

	Gtk::Label* label = manage(
		new Gtk::Label(Util::compose("<b>%s</b>", tConfigItem.stLabel.c_str()), 0.0, 0.0));
		
	label->set_alignment(1);
	label->set_use_markup(true);
	label->set_line_wrap((tConfigItem.stLabel.size() > LINE_WRAP_LEN) ? true : false);
	label->set_selectable(true);
	
	vbox4->pack_start(*label, false, false);
	
	Gtk::VBox* vbox1 = manage(new Gtk::VBox(false, 2));
	vbox4->pack_start(*vbox1);

	Gtk::Label* label2 = manage(new Gtk::Label(stMountSrc, 0.0, 0.0));
	label2->set_alignment(1);
	label2->set_use_markup(true);
	label2->set_line_wrap((stMountSrc.size() > LINE_WRAP_LEN) ? true : false);
	label2->set_selectable(true);
	
	vbox1->pack_start(*label2, false, false);

	Gtk::Label* label3 = manage(new Gtk::Label(stMountDir, 0.0, 0.0));
	label3->set_alignment(1);
	label3->set_use_markup(true);
	label3->set_line_wrap((stMountDir.size() > LINE_WRAP_LEN) ? true : false);
	label3->set_selectable(true);
	
	vbox1->pack_start(*label3, false, false);
	
	Gtk::HSeparator* hseparator = manage(new Gtk::HSeparator());
	vbox->pack_start(*hseparator);

	//
	// mid block
	//
	
	// show message
	if (!tConfigItem.stMsg.empty())
	{
		Gtk::Label* label4 = manage(new Gtk::Label(tConfigItem.stMsg, 0.5, 0.5));
		
		label4->set_use_markup(true);
		label4->set_line_wrap(true);
		label4->set_selectable(true);
		
		vbox->pack_start(*label4, false, false);
	}
	
	if (tConfigItem.bAskPassword)
	{
		Gtk::Alignment *align = manage(new Gtk::Alignment());
		align->set(0.5, 0.5, 0, 0);
		
		Gtk::HBox* hbox2 = manage(new Gtk::HBox(false, 6));
		align->add(*hbox2);
		vbox->pack_start(*align);
	
		Gtk::Label* label4 = manage(new Gtk::Label(_("Password:"), 0.0, 0.5));
		label4->set_alignment(1);
		label4->grab_focus();
		bFocusSet = true;
		hbox2->pack_start(*label4, false, false);
				
		gpoPasswdEntry = manage(new Gtk::Entry());
		gpoPasswdEntry->set_visibility(false);
		gpoPasswdEntry->set_size_request(120, -1);
		
		gpoPasswdEntry->signal_activate().connect(
			sigc::mem_fun(*this, &VolumeDialog::onPasswordActivate));

		hbox2->pack_start(*gpoPasswdEntry, false, false);

		// add cancel button
		add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
			
		bCancelButton = true;
	}
	else if (tConfigItem.bAskOverwrite)
	{
		Gtk::Alignment *align = manage(new Gtk::Alignment());
		align->set(0.5, 0.5, 0, 0);
		
		Gtk::HBox* hbox2 = manage(new Gtk::HBox(false, 6));
		align->add(*hbox2);
		vbox->pack_start(*align);
	
		Gtk::Label* label4 = manage(new Gtk::Label(_("Save As:"), 0.0, 0.5));
		label4->set_alignment(1);
		label4->grab_focus();
		bFocusSet = true;
		hbox2->pack_start(*label4, false, false);
				
		gpoSaveAsEntry = manage(new Gtk::Entry());
		gpoSaveAsEntry->set_text(tConfigItem.stOverwriteData);
		
		hbox2->pack_start(*gpoSaveAsEntry, false, false);

		Gtk::Button* button_browse = manage(new Gtk::Button(_("Browse...")));
		button_browse->signal_clicked().connect(
			sigc::mem_fun(*this, &VolumeDialog::onBrowseSaveAs));
		
		hbox2->pack_start(*button_browse, false, false);
		
		gpoOverwriteCheckButton = manage(new Gtk::CheckButton(_("Overwrite")));
		gpoOverwriteCheckButton->set_active(tConfigItem.bOverwrite);
		
		hbox2->pack_start(*gpoOverwriteCheckButton, false, false);
		
		// add cancel button
		add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
			
		bCancelButton = true;
	}
	else if (tConfigItem.bAskSelectDir)
	{
		Gtk::Alignment *align = manage(new Gtk::Alignment());
		align->set(0.5, 0.5, 0, 0);
		
		Gtk::HBox* hbox2 = manage(new Gtk::HBox(false, 6));
		align->add(*hbox2);
		vbox->pack_start(*align);
	
		Gtk::Label* label4 = manage(new Gtk::Label(tConfigItem.stSelectDirFieldName, 0.0, 0.5));
		label4->set_alignment(1);
		label4->grab_focus();
		bFocusSet = true;
		hbox2->pack_start(*label4, false, false);
				
		gpoSelectDirEntry = manage(new Gtk::Entry());
		
		hbox2->pack_start(*gpoSelectDirEntry, false, false);

		Gtk::Button* button_browse = manage(new Gtk::Button(_("Browse...")));
		button_browse->signal_clicked().connect(
			sigc::mem_fun(*this, &VolumeDialog::onBrowseSelectDir));
		
		hbox2->pack_start(*button_browse, false, false);
		
		Gtk::ToggleButton* button_perm = manage(new Gtk::ToggleButton(_("Ownership & Permissions")));
		button_perm->signal_released().connect(
			sigc::mem_fun(*this, &VolumeDialog::onPermissions));
		
		hbox2->pack_start(*button_perm, false, false);
		
		gpoPermToggleButton = button_perm;
		
		// add cancel button
		add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
			
		bCancelButton = true;
	}
	
	Gtk::HSeparator* hseparator2 = manage(new Gtk::HSeparator());
	vbox->pack_start(*hseparator2);
	
	//
	// command details
	//
	if (!tConfigItem.stCommand.empty())
	{
		Gtk::Frame* frame = manage(new Gtk::Frame(_("Details")));
		vbox->pack_start(*frame);
	
		Gtk::VBox* vbox2 = manage(new Gtk::VBox(false, 0));
		vbox2->set_border_width(6);
		frame->add(*vbox2);

		Gtk::VBox* vbox3 = manage(new Gtk::VBox(false, 2));
		vbox2->pack_start(*vbox3, false, false);

		Gtk::Label* label5 = manage(
			new Gtk::Label(Util::compose(
				"Source: %s", (tConfigItem.stSrc.empty()) ? _("n/a") : tConfigItem.stSrc.c_str()), 0.0, 0.0));
		
		label5->set_use_markup(true);
		label5->set_line_wrap((tConfigItem.stSrc.size() > LINE_WRAP_LEN) ? true : false);
		label5->set_selectable(true);
		
		vbox3->pack_start(*label5, false, false);

		Gtk::Label* label7 = manage(
			new Gtk::Label(Util::compose("Command: %s", tConfigItem.stCommand.c_str()), 0.0, 0.0));
		
		label7->set_use_markup(true);
		label7->set_line_wrap(true);
		label7->set_selectable(true);
		
		vbox3->pack_start(*label7, false, false);
	}
	
	// cancel button
	if (!bCancelButton && tConfigItem.bCancelButton)
	{
		Gtk::Button *button = 0;
		
		(tConfigItem.bYesNo) ?
			button = add_button(Gtk::Stock::NO, Gtk::RESPONSE_CANCEL) :
			button = add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
		
		if (tConfigItem.bCancelButtonHasFocus)
		{
			button->grab_focus();
			bFocusSet = true;
		}	
	}
	
	// ok button
	Gtk::Button *button2 = 0;
	
	(tConfigItem.bYesNo) ?
		button2 = add_button(Gtk::Stock::YES, Gtk::RESPONSE_OK) :
		button2 = add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
	
	if (!bFocusSet)
	{
		button2->grab_focus();
		bFocusSet = true;
	}
	
	show_all();
	
	if (tConfigItem.bAutoRun)
		run();
}

//==============================================================================

VolumeDialog::~VolumeDialog()
{
}

//==============================================================================

const Glib::ustring VolumeDialog::getOverwriteData()
{
	if (gpoSaveAsEntry)
		return gpoSaveAsEntry->get_text();
	
	return "";
}

//==============================================================================

const bool VolumeDialog::getOverwriteFlag()
{
	if (gpoOverwriteCheckButton)
		return gpoOverwriteCheckButton->get_active();
		
	return false;
}

//==============================================================================

const Glib::ustring VolumeDialog::getPassword()
{
	if (gpoPasswdEntry && !gpoPasswdEntry->get_text().empty())
	{
		Glib::ustring st64Enc;
					
		if (Util::base64Enc(gpoPasswdEntry->get_text(), &st64Enc))
			return st64Enc;
	}
	
	return Glib::ustring();
}

//==============================================================================

QDialog::tConfig VolumeDialog::getQDialogConfig(const tConfig& tConfigItem)
{
	QDialog::tConfig tQDialogConfigItem;
	QDialog::initConfig(tQDialogConfigItem);
	
	tQDialogConfigItem.stTitle = tConfigItem.stLabel;
	tQDialogConfigItem.bWaitData = tConfigItem.bWaitData;
	
	return tQDialogConfigItem;
}

//==============================================================================

const Glib::ustring VolumeDialog::getSelectDir()
{
	return (gpoSelectDirEntry) ? gpoSelectDirEntry->get_text() : "";
}

//==============================================================================

void VolumeDialog::initConfig(tConfig& tConfigItem)
{
	tConfigItem.bAutoRun = true;
	tConfigItem.bWaitData = true;
	tConfigItem.bAskPassword = false;
	tConfigItem.bAskOverwrite = false;
	tConfigItem.stOverwriteData.clear();
	tConfigItem.bOverwrite = false;
	tConfigItem.bAskSelectDir = false;
	tConfigItem.stSelectDirFieldName.clear();
	tConfigItem.bCancelButton = false;
	tConfigItem.bCancelButtonHasFocus = false;
	tConfigItem.bYesNo = false;
	tConfigItem.stLabel.clear();
	tConfigItem.stMsg.clear();
	tConfigItem.stSrc.clear();
	tConfigItem.stCommand.clear();
}

//==============================================================================

void VolumeDialog::onBrowseSaveAs()
{
	assert(gpoSaveAsEntry);
	
	Gtk::FileChooserDialog oDlg(*this, _("Save As"), Gtk::FILE_CHOOSER_ACTION_SAVE);
	
	oDlg.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	oDlg.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_OK);
	
	if (gpoSaveAsEntry->get_text().empty())
	{
		// set default path
		oDlg.set_current_folder(
			QConfig::getInstance().getDfCloneDir());
	}
	else
	{
		oDlg.set_current_folder(
			Glib::path_get_dirname(gpoSaveAsEntry->get_text()));
	}

	if (oDlg.run() == Gtk::RESPONSE_OK)
	{
		gpoSaveAsEntry->set_text(
			oDlg.get_filename());
			
		gpoSaveAsEntry->set_position(
			oDlg.get_filename().size());
	}
}

//==============================================================================

void VolumeDialog::onBrowseSelectDir()
{
	assert(gpoSelectDirEntry);
	
	Gtk::FileChooserDialog oDlg(*this, _("Select Directory"), Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER);
	
	oDlg.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	oDlg.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
	
	if (gpoSelectDirEntry->get_text().empty())
	{
		// set default path
		oDlg.set_current_folder(
			QConfig::getInstance().getDfDir());
	}
	else
	{
		oDlg.set_current_folder(
			Glib::path_get_dirname(gpoSelectDirEntry->get_text()));
	}

	if (oDlg.run() == Gtk::RESPONSE_OK)
	{
		gpoSelectDirEntry->set_text(
			oDlg.get_filename());
			
		gpoSelectDirEntry->set_position(
			oDlg.get_filename().size());
	}
}
//==============================================================================

void VolumeDialog::onPasswordActivate()
{
	if (getPassword().empty())
		return;
		
	Gtk::Dialog::response(Gtk::RESPONSE_OK);
}

//==============================================================================

void VolumeDialog::onPermissions()
{
	assert(gpoMountData);
	
	PermDialog *poPermDlg = 0;
	
	eMountSys::getInstance().getGuiBuilder()->get_widget_derived("PermDialog", poPermDlg);
	
	if (!poPermDlg)
		return;
	
	eMountWin::addOpenedDialog(poPermDlg);
	
	poPermDlg->set_transient_for(*this);
	poPermDlg->setData(gpoMountData);

	if (poPermDlg->run() == Gtk::RESPONSE_OK)
		gpoPermToggleButton->set_active(gpoMountData->getDirSetPerm());
	
	poPermDlg->hide();
}

//==============================================================================
