/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef VOLTREEVIEW_H
#define VOLTREEVIEW_H

#include <gtkmm.h>
#include <glibmm/timer.h>

class VolumeCellRenderer;
class VolumeGuiState;

class VolumeTreeView : public Gtk::TreeView
{
public:
	VolumeTreeView(Gtk::ScrolledWindow *poScrollWin);
	~VolumeTreeView();
		
	// volumes
	void addVolume(	VolumeGuiState *poVolumeGuiState,
					const bool bSelectVolume = true);
	void removeVolume(VolumeGuiState *poVolumeGuiState);
	void updateVolume(VolumeGuiState *poVolumeGuiState);
	
	VolumeGuiState *getSelectedVolume();
	
	// sort
	void sortByLabel();
	void sortByPos();
	
	//
	// signals
	//
	
	// buttons
	inline sigc::signal<void, VolumeGuiState*>& signal_cancel() {return m_signal_cancel;}
	inline sigc::signal<void, VolumeGuiState*>& signal_eject() {return m_signal_eject;}
	inline sigc::signal<void, VolumeGuiState*>& signal_key_enter_pressed() {return m_signal_key_enter_pressed;}
	inline sigc::signal<void, VolumeGuiState*>& signal_mount() {return m_signal_mount;}
	inline sigc::signal<void, VolumeGuiState*, GdkEventButton*>& 
		signal_popup_menu() {return m_signal_popup_menu;}
	
	// other
	inline sigc::signal<void, VolumeGuiState*>& signal_dclick()
		{return m_signal_dclick;}
	inline sigc::signal<void, VolumeGuiState*>& signal_selection_changed()
		{return m_signal_selection_changed;}
	
protected:
	// override events
	virtual bool on_button_press_event(GdkEventButton* event);
	virtual bool on_key_press_event(GdkEventKey* event);
	virtual bool on_motion_notify_event(GdkEventMotion* event);
	
	// VolumeCellRenderer events
	virtual void on_cell_activate(  const int iIndex,
									const int ix,
									const int iy);
											
	// TreeView signals
	virtual void on_rows_reordered( const Gtk::TreeModel::Path& path, 
									const Gtk::TreeModel::iterator& iter, 
									int* new_order);
	  	
private:
	// parent window
	Gtk::ScrolledWindow *gpoScrollWin;
	
	// volumes
	typedef struct _VOLUMEGUISTATE_
	{
		int iIndex;
		VolumeGuiState* poVolumeGuiState;
	}
	tVolumeGuiState;
	
	std::list<tVolumeGuiState> glstVolumeGuiState;

	void mapVolumeGuiState( Gtk::TreeModel::Row& oRow,
							VolumeGuiState *poVolumeGuiState);

	// sort
	Glib::RefPtr<Gtk::TreeModelSort> grefSortedModel;
	
	void sortVolumeCellRenderer();
	
	// utility
	bool getRowByIndex(const int iIndex, Gtk::TreeModel::Row *poRow);
	VolumeGuiState *getVolumeByIndex(const int iIndex);
	
	// signals
	Glib::Timer goTimerActivate; // start time since activate has been clicked stop at 10 sec
	sigc::connection goSigTimeoutConn; // reset goTimerActivate timer
	
	sigc::signal<void, VolumeGuiState*>  m_signal_cancel;
	sigc::signal<void, VolumeGuiState*>  m_signal_eject;
	sigc::signal<void, VolumeGuiState*>  m_signal_key_enter_pressed;
	sigc::signal<void, VolumeGuiState*>  m_signal_mount;
	sigc::signal<void, VolumeGuiState*, GdkEventButton*>  m_signal_popup_menu;
	
	sigc::signal<void, VolumeGuiState*> m_signal_dclick;
	sigc::signal<void, VolumeGuiState*> m_signal_selection_changed;
	
	bool on_timeout();
	
	// cell renderer
	VolumeCellRenderer				*gpoVolumeCellRenderer;
	
	// ModelColumns
	class ModelColumns : public Gtk::TreeModel::ColumnRecord
	{
	public:
		ModelColumns();
		
		Gtk::TreeModelColumn<VolumeCellRenderer*>   goColVolumeCellRenderer;
		Gtk::TreeModelColumn<int>					goColIndex;
		Gtk::TreeModelColumn<int>					goColHeight;
		Gtk::TreeModelColumn<int>					goColPos;
		Gtk::TreeModelColumn<int>					goColBtnCancel;
		Gtk::TreeModelColumn<int>					goColBtnEject;
		Gtk::TreeModelColumn<int>					goColBtnMount;
		Gtk::TreeModelColumn<Glib::ustring>			goColDrawInfo;
		Gtk::TreeModelColumn<Glib::ustring>			goColDrawLabel;
		Gtk::TreeModelColumn<bool>					goColDrawProgressFull;
		Gtk::TreeModelColumn<int>					goColDrawProgressPctGauge;
		Gtk::TreeModelColumn<int>					goColDrawProgressPctUsed;
	};
	Glib::RefPtr<Gtk::ListStore>	grefTreeModel;
	ModelColumns					goColumns;
	
	// gui status
	typedef struct _GUISTATE_
	{
		int iActiveIndex;
	}
	tGuiState;
	tGuiState gtGuiState;
	
	// selections
	void on_selection_changed();
	void selected_row_callback(const Gtk::TreeModel::iterator& ite);
};
	
#endif /* VOLTREEVIEW_H */
