/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "eMountWin.h"
#include "QDebug.h"
#include "QConfig.h"
#include "../config.h"
#include "Icon.h"
#include "eMountSys.h"

#include "DispatchData.h"
#include "VolumeGuiState.h"
#include "VolumeMountGui.h"
#include "ThreadVolumeStatus.h"
#include "Util.h"

#include "Automount.h"

#include "Pline/MountUtil.h"
#include "Pline/CryptUtil.h"
#include "Pline/FSysUtil.h"

#include "CreateDiskImageDialog.h"
#include "CreatePlainDiskImageDialog.h"
#include "CreateVolumeDialog.h"
#include "EnlargeVolumeDialog.h"
#include "InitVolumeDialog.h"
#include "MountDialog.h"
#include "MsgDialog.h"
#include "ProgressDialog.h"
#include "PropDialog.h"

#include <glibmm/i18n.h>
#include <assert.h>

using Gtk::Menu_Helpers::MenuElem;
using Gtk::Menu_Helpers::CheckMenuElem;
using Gtk::Menu_Helpers::RadioMenuElem;
using Gtk::Menu_Helpers::SeparatorElem;

eMountWin* eMountWin::gs_window = NULL;

//==============================================================================

eMountWin::eMountWin(   BaseObjectType* cobject,
						const Glib::RefPtr<Gtk::Builder>& refGlade)
: Gtk::Window(cobject),
  gpoRefGlade(refGlade)
{	
	// init members
	gs_window = this;
	gpoVolumeTreeView = 0;
	gbUserIdle = true;
	
	gpoThreadVolumeStatus = 0;
	gpoAutomount = 0;
	
	hide();
	
	QConfig &oCfg = QConfig::getInstance();

	// init win pos/size var
	giWinPosX = 0;
	giWinPosY = 0;
	giWinSizeX = 250;
	giWinSizeY = 450;
	
	// position main window
	if (oCfg.isSaveWinPos()
		&& oCfg.getMainWinX(&giWinPosX) 
		&& oCfg.getMainWinY(&giWinPosY)
		&& giWinPosX < Gdk::Screen::get_default()->get_width()
		&& giWinPosY < Gdk::Screen::get_default()->get_height())
	{
		move(giWinPosX, giWinPosY);
	}
	
	// resize main window
	if (oCfg.isSaveWinSize()
		&& oCfg.getMainWinW(&giWinSizeX) 
		&& oCfg.getMainWinH(&giWinSizeY)
		&& giWinSizeX <= Gdk::Screen::get_default()->get_width() - giWinSizeX 
		&& giWinSizeY <= Gdk::Screen::get_default()->get_height() - giWinSizeY)
	{
		resize(giWinSizeX, giWinSizeY);
	}
	else
		resize(giWinSizeX, giWinSizeY);
		
	// build the menubar
	buildMenuBar();
	
	// signals
	signal_configure_event().connect_notify(
			sigc::mem_fun(*this, &eMountWin::onWinResize));
	signal_delete_event().connect(
			sigc::mem_fun(*this, &eMountWin::onWinDelete));
			
	// add volumes treeview
	Gtk::ScrolledWindow *poVolumeTreeViewScrolledWin;
	gpoRefGlade->get_widget("VolumeTreeViewScrolledWindow", poVolumeTreeViewScrolledWin);
	
	if (!poVolumeTreeViewScrolledWin)
	{
		enableVolumeMenu(false);
		show_all_children();
		return;
	}
	
	//
	// volumes
	//
	
	// treeview
	gpoVolumeTreeView = new VolumeTreeView(poVolumeTreeViewScrolledWin);
	
	// connect signals
	gpoVolumeTreeView->signal_cancel().connect(
			sigc::mem_fun(*this, &eMountWin::onVolumeCancel));
		
	gpoVolumeTreeView->signal_dclick().connect(
			sigc::mem_fun(*this, &eMountWin::onVolumeDClick));
				
	gpoVolumeTreeView->signal_eject().connect(
			sigc::mem_fun(*this, &eMountWin::onVolumeEject));
	
	gpoVolumeTreeView->signal_key_enter_pressed().connect(
			sigc::mem_fun(*this, &eMountWin::onVolumeEnterPressed));
	
	gpoVolumeTreeView->signal_mount().connect(
			sigc::mem_fun(*this, &eMountWin::onVolumeMount));
	
	gpoVolumeTreeView->signal_popup_menu().connect(
			sigc::mem_fun(*this, &eMountWin::onVolumePopupMenu));
			
	gpoVolumeTreeView->signal_selection_changed().connect(
			sigc::mem_fun(*this, &eMountWin::onVolumeSelectionChanged));
				
	poVolumeTreeViewScrolledWin->add(*gpoVolumeTreeView);
	
	// show VolumeGui	
	std::list<VolumeConfig*> lstVolumeConfig = oCfg.getVolumes();
		
	for (   std::list<VolumeConfig*>::iterator ite = lstVolumeConfig.begin();
			ite != lstVolumeConfig.end();
			++ite)
	{
		Glib::Mutex::Lock lock(gmtxVolumeGuiLst);
		
		VolumeGui *poVolumeGui = new VolumeMountGui(*ite);
		
		assert(poVolumeGui->getVolumeGuiState());
		
		gpoVolumeTreeView->addVolume(poVolumeGui->getVolumeGuiState(), false);
		
		glstVolumeGui.push_back(poVolumeGui);
	}
	
	// sort treeview
	if (((Gtk::RadioMenuItem&)goViewMenu.items()[kMenuViewSortByLabel]).get_active())
		gpoVolumeTreeView->sortByLabel();
	else if (((Gtk::RadioMenuItem&)goViewMenu.items()[kMenuViewSortByPos]).get_active())
		gpoVolumeTreeView->sortByPos();
		
	// enable Volume menu
	enableVolumeMenu((gpoVolumeTreeView->getSelectedVolume()) ? true : false);
	
	// drag and drop
	std::list<Gtk::TargetEntry> listTargets;
	listTargets.push_back( Gtk::TargetEntry("STRING") );
	listTargets.push_back( Gtk::TargetEntry("text/plain") );

	this->drag_dest_set(listTargets);

	this->signal_drag_data_received().connect(sigc::mem_fun(*this,
		&eMountWin::on_drop_drag_data_received) );

	// show window
	show_all_children();
	
	// start mount status thread
	Glib::signal_idle().connect(
		sigc::bind_return(sigc::mem_fun(*this, &eMountWin::launchVolumeStatusThread), false));
	
	// start automount
	if (!QConfig::getInstance().isDisableAutomount())
		Glib::signal_idle().connect(
			sigc::bind_return(sigc::mem_fun(*this, &eMountWin::launchAutomount), false));
}

//==============================================================================

eMountWin::~eMountWin()
{
	assert(!gpoThreadVolumeStatus);
	
	if (gpoAutomount)
		delete gpoAutomount;
		
	// Dialogs
	for (	std::vector<Gtk::Dialog*>::size_type xi = 0;
			xi < gvecOpenedDialog.size();
			++xi)
	{
		delete gvecOpenedDialog[xi];
	}
	
	// VolumeTreeView
	if (gpoVolumeTreeView)
		delete gpoVolumeTreeView;
	
	// Volumes
	for (   std::list<VolumeGui*>::iterator ite = glstVolumeGui.begin();
			ite != glstVolumeGui.end();
			++ite)
	{
		delete *ite;
	}
	
	// Disable Automount
	QConfig::getInstance().setDisableAutomount(
		((Gtk::CheckMenuItem&)goFileMenu.items()[kMenuFileDisableAutomount]).get_active());
	
	// save win pos/size	
	if (((Gtk::CheckMenuItem&)goViewMenu.items()[kMenuViewSaveWindowPos]).get_active())
	{
		QConfig::getInstance().setSaveWinPos(true);
		QConfig::getInstance().setMainWinX(giWinPosX);
		QConfig::getInstance().setMainWinY(giWinPosY);
	}
	else
		QConfig::getInstance().setSaveWinPos(false);

	if (((Gtk::CheckMenuItem&)goViewMenu.items()[kMenuViewSaveWindowSize]).get_active())
	{
		QConfig::getInstance().setSaveWinSize(true);
		QConfig::getInstance().setMainWinW(giWinSizeX);
		QConfig::getInstance().setMainWinH(giWinSizeY);
	}
	else
		QConfig::getInstance().setSaveWinSize(false);
}

//==============================================================================

void eMountWin::addOpenedDialog(Gtk::Dialog *po)
{
	bool bFound = false;
	
	eMountWin *poEMountWin = eMountWin::getInstance();
	
	if (!poEMountWin || !po)
		return;
		
	for (	std::vector<Gtk::Dialog*>::size_type xi = 0;
			xi < poEMountWin->gvecOpenedDialog.size();
			++xi)
	{
		if (poEMountWin->gvecOpenedDialog[xi] == po)
		{
			bFound = true;
			break;
		}
	}
	
	if (!bFound)
		poEMountWin->gvecOpenedDialog.push_back(po);
}

//==============================================================================

VolumeMountGui* eMountWin::addVolume(VolumeConfig* poVolumeConfig)
{
	Glib::Mutex::Lock lock(gmtxVolumeGuiLst);
	
	// set position
	std::list<VolumeConfig*> lstVolumeConfig = QConfig::getInstance().getVolumes();
	std::list<VolumeConfig*>::iterator ite_insert;

	if (poVolumeConfig->getData().getPos() == 0)
		lstVolumeConfig.push_front(poVolumeConfig);
	else
	{
		lstVolumeConfig.sort(VolumeConfig::cmpByPos);
		
		for (   ite_insert = lstVolumeConfig.begin();
				ite_insert != lstVolumeConfig.end();
				++ite_insert)
		{
			if (((VolumeConfig*)*ite_insert)->getData().getPos() 
				== poVolumeConfig->getData().getPos())
			{
				break;
			}
		}
		
		lstVolumeConfig.insert(ite_insert, poVolumeConfig);
	}
	
	// add volume to config
	QConfig::getInstance().addVolume(poVolumeConfig);
	
	VolumeMountGui *poVolumeGui = new VolumeMountGui(poVolumeConfig);
	
	assert(poVolumeGui->getVolumeGuiState());
	
	if (gpoVolumeTreeView)
		gpoVolumeTreeView->addVolume(poVolumeGui->getVolumeGuiState());
	
	glstVolumeGui.push_back(poVolumeGui);
	
	updateVolumeConfigPositions(lstVolumeConfig);
	
	return poVolumeGui;
}

//==============================================================================

void eMountWin::buildMenuBar()
{
	Gtk::VBox *poVBox = 0;
	
	// File->New
	goFileNewMenu.items().push_back(MenuElem(_("_Plain Disk Image..."), 
											Gtk::AccelKey("<control><shift>N"),
											sigc::mem_fun(*this, &eMountWin::onCreatePlainDiskImage)
											));

	goFileNewMenu.items().push_back(MenuElem(_("_Encrypted Disk Image..."), 
											Gtk::AccelKey("<control>N"),
											sigc::mem_fun(*this, &eMountWin::onCreateDiskImage)
											));
	
	goFileNewMenu.items().push_back(Gtk::Menu_Helpers::SeparatorElem());
	
	goFileNewMenu.items().push_back(MenuElem(_("E_ncrypted Partition..."), 
											sigc::mem_fun(*this, &eMountWin::onCreateVolume)
											));
	
	// File
	goFileMenu.items().push_back(MenuElem(_("_New"), goFileNewMenu));
	
	goFileMenu.items().push_back(Gtk::Menu_Helpers::SeparatorElem());
	
	goFileMenu.items().push_back(MenuElem(_("_Mount..."), 
											Gtk::AccelKey("<control><shift>M"),
											sigc::mem_fun(*this, &eMountWin::onMount)
											));
	
	goFileMenu.items().push_back(Gtk::Menu_Helpers::SeparatorElem());
	
	goFileMenu.items().push_back(MenuElem(_("Load _Ciphers"),
											sigc::mem_fun(*this, &eMountWin::onLoadCipher)
											));
	goFileMenu.items().push_back(MenuElem(_("Load _File Systems"),
											sigc::mem_fun(*this, &eMountWin::onLoadFSys)
											));
	
	goFileMenu.items().push_back(Gtk::Menu_Helpers::SeparatorElem());
	
	
	goFileMenu.items().push_back(MenuElem(_("_Eject All"),
											Gtk::AccelKey("<control><shift>E"),
											sigc::mem_fun(*this, &eMountWin::onEjectAll)
											));
	
	goFileMenu.items().push_back(CheckMenuElem(_("_Disable Automount")));
		
	((Gtk::CheckMenuItem&)goFileMenu.items()[kMenuFileDisableAutomount]).
		set_active(QConfig::getInstance().isDisableAutomount());
		
	goFileMenu.items().push_back(Gtk::Menu_Helpers::SeparatorElem());
	
	goFileMenu.items().push_back(MenuElem(_("_Quit"), 
											Gtk::AccelKey("<control>Q"),
											sigc::mem_fun(*this, &eMountWin::onQuit)
											));
	
	// Volume
	goVolumeMenu.items().push_back(MenuElem(_("_Mount"),
												Gtk::AccelKey("<control>M"),
												sigc::mem_fun(*this, &eMountWin::onSelVolumeMount)
												));
	
	goVolumeMenu.items().push_back(MenuElem(_("_Eject"),
												Gtk::AccelKey("<control>E"),
												sigc::mem_fun(*this, &eMountWin::onSelVolumeEject)
												));
												
	goVolumeMenu.items().push_back(Gtk::Menu_Helpers::SeparatorElem());
	
	goVolumeMenu.items().push_back(MenuElem(_("_Clone..."),
												sigc::mem_fun(*this, &eMountWin::onSelVolumeClone)
												));
	
	goVolumeMenu.items().push_back(MenuElem(_("E_nlarge..."),
												sigc::mem_fun(*this, &eMountWin::onSelVolumeEnlarge)
												));
	
	goVolumeMenu.items().push_back(MenuElem(_("_Initialize..."),
												sigc::mem_fun(*this, &eMountWin::onSelVolumeInit)
												));
												
	goVolumeMenu.items().push_back(Gtk::Menu_Helpers::SeparatorElem());
																							
	goVolumeMenu.items().push_back(MenuElem(_("_Remove"),
												sigc::mem_fun(*this, &eMountWin::onSelVolumeRemove)
												));
	
	goVolumeMenu.items().push_back(Gtk::Menu_Helpers::SeparatorElem());
	
	goVolumeMenu.items().push_back(MenuElem(_("_Properties"),
												Gtk::AccelKey("<control><shift>P"),
												sigc::mem_fun(*this, &eMountWin::onSelVolumeProp)
												));
	
	// View
	Gtk::RadioButtonGroup oSortGroup;
	
	goViewMenu.items().push_back(RadioMenuElem( oSortGroup, _("Sort by _Label"),
												Gtk::AccelKey("<control>L"),
												sigc::bind<1>(
													sigc::mem_fun(*this, &eMountWin::onSortBy)
													, (int)kMenuViewSortByLabel)
												));
	
	goViewMenu.items().push_back(RadioMenuElem( oSortGroup, _("Sort by _Position"),
												Gtk::AccelKey("<control>P"),
												sigc::bind<1>(
													sigc::mem_fun(*this, &eMountWin::onSortBy)
													, (int)kMenuViewSortByPos)
												));
	
	int iSortID = 0;
	
	if (QConfig::getInstance().getSortBy(&iSortID))
	{
		switch (iSortID)
		{
			case kSortByPos:
				((Gtk::CheckMenuItem&)goViewMenu.items()[kMenuViewSortByPos]).
					set_active(true);
				break;
			case kSortByLabel:
				((Gtk::CheckMenuItem&)goViewMenu.items()[kMenuViewSortByLabel]).
					set_active(true);
				break;
			default:
				((Gtk::CheckMenuItem&)goViewMenu.items()[kMenuViewSortByPos]).
				set_active(true);
				break;
		}
	}
	else
	{
		// default to sort by position
		((Gtk::CheckMenuItem&)goViewMenu.items()[kMenuViewSortByPos]).
			set_active(true);
	}
	
	goViewMenu.items().push_back(Gtk::Menu_Helpers::SeparatorElem());
	
	goViewMenu.items().push_back(CheckMenuElem(_("Save Window Position")));
	goViewMenu.items().push_back(CheckMenuElem(_("Save Window Geometry")));
		
	((Gtk::CheckMenuItem&)goViewMenu.items()[kMenuViewSaveWindowPos]).
		set_active(QConfig::getInstance().isSaveWinPos());
	((Gtk::CheckMenuItem&)goViewMenu.items()[kMenuViewSaveWindowSize]).
		set_active(QConfig::getInstance().isSaveWinSize());

	// HelperList
	goHelpMenu.items().push_back(MenuElem(_("_About..."),
											Gtk::AccelKey("F1"),
											sigc::mem_fun(*this, &eMountWin::onAbout)
											));
												
	goMenuBar.items().push_back(MenuElem(_("_File"), goFileMenu));
	goMenuBar.items().push_back(MenuElem(_("Vol_ume"), goVolumeMenu));
	goMenuBar.items().push_back(MenuElem(_("_View"), goViewMenu));
	goMenuBar.items().push_back(MenuElem(_("_Help"), goHelpMenu));

	gpoRefGlade->get_widget("MainMenuVBox", poVBox);

	if (poVBox)
	{
		poVBox->pack_start(goMenuBar, Gtk::PACK_SHRINK);
		goMenuBar.show();
	}
}

//==============================================================================

void eMountWin::enableVolumeMenu(const bool bEnable)
{
	goVolumeMenu.set_sensitive(bEnable);
}

//==============================================================================

VolumeGui* eMountWin::findVolume(VolumeGuiState* poVolumeGuiState)
{
	Glib::Mutex::Lock lock(gmtxVolumeGuiLst);

	for (   std::list<VolumeGui*>::iterator ite = glstVolumeGui.begin();
			ite != glstVolumeGui.end();
			++ite)
	{
		if (poVolumeGuiState == ((VolumeGui*)*ite)->getVolumeGuiState())
			return *ite;
	}
	
	return 0;
}

//==============================================================================

VolumeConfig* eMountWin::findVolumeConfig(VolumeGuiState* poVolumeGuiState)
{
	VolumeGui* poVolumeGui = 0;
	
	if (!poVolumeGuiState)
		return 0;
	
	return ((poVolumeGui = findVolume(poVolumeGuiState))) ? &poVolumeGui->getVolumeConfig() : 0;
}

//==============================================================================

eMountWin* eMountWin::getInstance()
{
	g_return_val_if_fail(gs_window, NULL);

	return gs_window;
}

//==============================================================================

void eMountWin::getVolumeConfigLabelList(std::vector<Glib::ustring> *pvecVolumeConfigLabel)
{
	Glib::Mutex::Lock lock(gmtxVolumeGuiLst);
	
	std::list<VolumeConfig*> lstVolumeConfig = QConfig::getInstance().getVolumes();

	lstVolumeConfig.sort(VolumeConfig::cmpByPos);
	
	for (   std::list<VolumeConfig*>::iterator ite = lstVolumeConfig.begin();
			ite != lstVolumeConfig.end();
			++ite)
	{
		VolumeConfig *poVolumeConfig = (VolumeConfig*)*ite;
				
		pvecVolumeConfigLabel->push_back(poVolumeConfig->getData().getLabel());
	}
}

//==============================================================================

bool eMountWin::hasDispatchData(void *p, int *piCount)
{
	eMountWin *poEMountWin = eMountWin::getInstance();
	
	if (piCount)
		*piCount = 0;
		
	if (!poEMountWin)
		return false;
	
	poEMountWin->gmtxDispatchData.lock();
		
	for (	std::list<DispatchData*>::iterator ite = poEMountWin->glstDispatchData.begin();
			ite != poEMountWin->glstDispatchData.end();
			++ite)
	{
		DispatchData *poDispatchData = (DispatchData*)*ite;
		
		if (poDispatchData->getCaller() == p)
		{
			poEMountWin->gmtxDispatchData.unlock();
			
			if (piCount)
				*piCount = *piCount + 1;
			else
			{
				poEMountWin->gmtxDispatchData.unlock();
				return true;
			}
		}
	}
	
	poEMountWin->gmtxDispatchData.unlock();
	
	if (piCount && *piCount > 0)
		return true;
		
	return false;
}

//==============================================================================

void eMountWin::launchAutomount()
{
	gpoAutomount = new Automount(glstVolumeGui, gmtxVolumeGuiLst);
	gpoAutomount->start();
}

//==============================================================================

void eMountWin::launchVolumeStatusThread()
{
	Glib::Mutex::Lock lock(gmtxVolumeGuiLst);

	if (gpoThreadVolumeStatus)
		return;
	
	gpoThreadVolumeStatus = new ThreadVolumeStatus(glstVolumeGui, gmtxVolumeGuiLst);	
	gpoThreadVolumeStatus->launch();
}

//==============================================================================

void eMountWin::mount(const Glib::ustring& stSrc)
{
	MountDialog *poMountDlg = 0;
	std::vector<Glib::ustring> vecVolumeConfigLabel;
	
	eMountSys::getInstance().getGuiBuilder()->get_widget_derived("MountDialog", poMountDlg);
	
	if (!poMountDlg)
		return;

	addOpenedDialog(poMountDlg);

	getVolumeConfigLabelList(&vecVolumeConfigLabel);

	VolumeConfig *poVolumeConfig = new VolumeConfig;
	
	Pline::Data *poData = new Pline::Data;
	poVolumeConfig->setData(poData);
	
	poMountDlg->setData(poVolumeConfig,
						&vecVolumeConfigLabel,
						stSrc);
		
	gmtxUserIdle.lock();
	
	if (poMountDlg->run() == Gtk::RESPONSE_OK)
	{
		VolumeMountGui *poVolumeGui = addVolume(poVolumeConfig);
		
		if (!poMountDlg->isAddOnly() && !poVolumeGui->isInUse())
			poVolumeGui->mount();
	}
	else
		delete poVolumeConfig;
		
	gmtxUserIdle.unlock();
		
	processQueueData();
}

//==============================================================================

void eMountWin::onAbout()
{
	Gtk::AboutDialog *poDlg  = new Gtk::AboutDialog;
	
	setChild(*poDlg);
	
	poDlg->set_name(QAPPNAME);
	poDlg->set_version(PACKAGE_VERSION);
	poDlg->set_comments(QSHORTDESC);
	poDlg->set_copyright(QCOPYRIGHT);
	poDlg->set_website_label(QWEBSITE);
	
	if (Icon::getInstance().drawSysIcon64())
		poDlg->set_logo(*Icon::getInstance().drawSysIcon64());
	
	gmtxUserIdle.lock();
	
	poDlg->run();
	delete poDlg;
	
	gmtxUserIdle.unlock();
	
	processQueueData();
}

//==============================================================================

void eMountWin::onCreateDiskImage()
{	
	CreateDiskImageDialog *poCreateDlg = 0;
	std::vector<Glib::ustring> vecVolumeConfigLabel;
		
	eMountSys::getInstance().getGuiBuilder()->get_widget_derived("CreateDiskImageDialog", poCreateDlg);
	
	if (!poCreateDlg)
		return;

	addOpenedDialog(poCreateDlg);

	getVolumeConfigLabelList(&vecVolumeConfigLabel);

	VolumeConfig *poVolumeConfig = new VolumeConfig;
	
	Pline::Data *poData = new Pline::Data;
	
	poVolumeConfig->setData(poData);
	
	unsigned long long ullVolumeSize = 0;
	bool bIsFullAllocation = false;
	Glib::ustring stCipher;
	Glib::ustring stCBC;
	Glib::ustring stKeySize;
	
	poCreateDlg->setData(   poVolumeConfig,
							&vecVolumeConfigLabel,
							&ullVolumeSize,
							&bIsFullAllocation,
							&stCipher,
							&stCBC,
							&stKeySize);
	
	gmtxUserIdle.lock();
	
	if (poCreateDlg->run() == Gtk::RESPONSE_OK)
	{
		VolumeMountGui *poVolumeGui = addVolume(poVolumeConfig);
		
		poVolumeGui->createDiskImage(	ullVolumeSize,
										bIsFullAllocation,
										stCipher,
										stCBC,
										stKeySize);
	}
	else
		delete poVolumeConfig;
		
	gmtxUserIdle.unlock();
	
	processQueueData();
}

//==============================================================================

void eMountWin::onCreatePlainDiskImage()
{		
	CreatePlainDiskImageDialog *poCreateDlg = 0;
	std::vector<Glib::ustring> vecVolumeConfigLabel;
		
	eMountSys::getInstance().getGuiBuilder()->get_widget_derived("CreatePlainDiskImageDialog", poCreateDlg);
	
	if (!poCreateDlg)
		return;

	addOpenedDialog(poCreateDlg);

	getVolumeConfigLabelList(&vecVolumeConfigLabel);

	VolumeConfig *poVolumeConfig = new VolumeConfig;
	
	Pline::Data *poData = new Pline::Data;
	poVolumeConfig->setData(poData);
	
	unsigned long long ullVolumeSize = 0;
	bool bIsFullAllocation = false;
	
	poCreateDlg->setData(   poVolumeConfig,
							&vecVolumeConfigLabel,
							&ullVolumeSize,
							&bIsFullAllocation);
	
	gmtxUserIdle.lock();
	
	if (poCreateDlg->run() == Gtk::RESPONSE_OK)
	{
		VolumeMountGui *poVolumeGui = addVolume(poVolumeConfig);
		
		poVolumeGui->createDiskImage(ullVolumeSize, bIsFullAllocation);
	}
	else
		delete poVolumeConfig;
		
	gmtxUserIdle.unlock();
	
	processQueueData();
}

//==============================================================================

void eMountWin::onCreateVolume()
{		
	CreateVolumeDialog *poCreateDlg = 0;
	std::vector<Glib::ustring> vecVolumeConfigLabel;
		
	eMountSys::getInstance().getGuiBuilder()->get_widget_derived("CreateVolumeDialog", poCreateDlg);
	
	if (!poCreateDlg)
		return;

	addOpenedDialog(poCreateDlg);

	getVolumeConfigLabelList(&vecVolumeConfigLabel);

	VolumeConfig *poVolumeConfig = new VolumeConfig;
	
	Pline::Data *poData = new Pline::Data;
	
	poVolumeConfig->setData(poData);
	
	Glib::ustring stCipher;
	Glib::ustring stCBC;
	Glib::ustring stKeySize;
	
	poCreateDlg->setData(   poVolumeConfig,
							&vecVolumeConfigLabel,
							&stCipher,
							&stCBC,
							&stKeySize);
	
	gmtxUserIdle.lock();
	
	if (poCreateDlg->run() == Gtk::RESPONSE_OK)
	{
		VolumeMountGui *poVolumeGui = addVolume(poVolumeConfig);
		
		poVolumeGui->createVolume(	stCipher,
									stCBC,
									stKeySize);
	}
	else
		delete poVolumeConfig;
		
	gmtxUserIdle.unlock();
	
	processQueueData();
}

//==============================================================================

void eMountWin::on_drop_drag_data_received(	const Glib::RefPtr<Gdk::DragContext>& context, 
											int, 
											int,
											const Gtk::SelectionData& selection_data, 
											guint, 
											guint time)
{
	const int length = selection_data.get_length();
	
	if ((length >= 0) && (selection_data.get_format() == 8))
	{
		Glib::ustring stSource = selection_data.get_data_as_string();
        
		Glib::ustring::size_type iPos;
        
        if ((iPos = stSource.find("file://")) != std::string::npos)
			stSource.erase(iPos, 7);
		
		if ((iPos = stSource.find("\n")) != std::string::npos)
			stSource = stSource.substr(0, iPos-1);
				
		while ((iPos = stSource.find("%20")) != std::string::npos) // un-escape spaces
			stSource.replace(iPos, 3, " ");

		if (Util::isDevice(stSource) || Util::isRegularFile(stSource))
			mount(stSource);
	}

	context->drag_finish(false, false, time);
}
											
//==============================================================================

void eMountWin::onEjectAll()
{
	int ciMounted = 0;
	int ciPrevMounted = 0;
	
	// Check if there are mounted volumes
	gmtxVolumeGuiLst.lock();
	
	ciMounted = 0;
	for (   std::list<VolumeGui*>::iterator ite = glstVolumeGui.begin();
			ite != glstVolumeGui.end();
			++ite)
	{
		VolumeMountGui* poVolumeGui = (VolumeMountGui*)*ite;
		bool bMounted = false;
		
		if (!poVolumeGui->lockThread())
			continue;
				
		if (poVolumeGui->isInUse(&bMounted) && bMounted)
			ciMounted++;
		
		poVolumeGui->lockThread(false);
	}
	
	gmtxVolumeGuiLst.unlock();
	
	if (ciMounted == 0)
	{
		MsgDialog::tConfig tConfigItem;
		MsgDialog::initConfig(tConfigItem);
		
		tConfigItem.bAutoRun = true;
		tConfigItem.stTitle = _("Eject All");
		tConfigItem.stMsg = "All volumes ejected.";
		
		MsgDialog oMsgDlg(tConfigItem);
	
		return;
	}
		
	// ask confirmation
	MsgDialog::tConfig tConfigItem;
	MsgDialog::initConfig(tConfigItem);
		
	tConfigItem.bAutoRun = false;
	tConfigItem.bYesNo = true;
	tConfigItem.stTitle = _("Eject All");
	tConfigItem.stMsg = (ciMounted > 1) ? 
		_("Eject all volumes?") : _("Eject volume?");
		
	MsgDialog oMsgDlg(tConfigItem);
	
	gmtxUserIdle.lock();
	
	if (oMsgDlg.run() != Gtk::RESPONSE_OK)
	{
		gmtxUserIdle.unlock();
		return;
	}
	
	gmtxUserIdle.unlock();
	
	// eject
	while (true)
	{
		ciPrevMounted = ciMounted;
		ciMounted = 0;
		
		gmtxVolumeGuiLst.lock();
		
		for (   std::list<VolumeGui*>::iterator ite = glstVolumeGui.begin();
				ite != glstVolumeGui.end();
				++ite)
		{
			VolumeMountGui* poVolumeGui = (VolumeMountGui*)*ite;
			bool bMounted = false;
		
			if (!poVolumeGui->lockThread())
				continue;
				
			if (poVolumeGui->isInUse(&bMounted) && bMounted)
			{
				ciMounted++;
				
				poVolumeGui->lockThread(false);
				
				poVolumeGui->eject();
			}
			else
				poVolumeGui->lockThread(false);
		}
		
		gmtxVolumeGuiLst.unlock();
		
		if (ciMounted == 0 || ciPrevMounted == ciMounted)
			break;
	}
}

//==============================================================================

void eMountWin::onLoadCipher()
{
	int ciLoaded = 0;
	std::vector<Glib::ustring> vecMod;
	
	if (!Pline::CryptUtil::getAvailCipherMod(&vecMod) || vecMod.size() == 0)
	{
		MsgDialog::tConfig tConfigItem;
		MsgDialog::initConfig(tConfigItem);
		
		tConfigItem.bAutoRun = true;
		tConfigItem.bWaitData = true;
		tConfigItem.stTitle = _("Load Ciphers");
		tConfigItem.stMsg = _("No modules found.");
		
		MsgDialog oDlg(tConfigItem);
		
		return;
	}
	
	// confirm operation
	MsgDialog::tConfig tConfigItemConfirm;
	MsgDialog::initConfig(tConfigItemConfirm);
		
	tConfigItemConfirm.bAutoRun = false;
	tConfigItemConfirm.bWaitData = true;
	tConfigItemConfirm.bYesNo = true;
	tConfigItemConfirm.stTitle = _("Load Ciphers");
	tConfigItemConfirm.stMsg = _("Ciphers modules will be loaded into the Linux Kernel, confirm?");
		
	MsgDialog oConfirmDlg(tConfigItemConfirm);
	
	if (oConfirmDlg.run() != Gtk::RESPONSE_OK)
		return;
	
	// load
	ProgressDialog::tConfig tConfigItem;
	ProgressDialog::initConfig(tConfigItem);
	
	tConfigItem.bAutoShow = true;
	tConfigItem.stTitle = _("Load Ciphers");
	tConfigItem.stMsg = _("Loading Ciphers...");
	
	ProgressDialog oDlg(tConfigItem);
	
	oDlg.set_modal(true);
	
	for (   std::vector<Glib::ustring>::size_type xi = 0;
			xi<vecMod.size();
			++xi)
	{	
		Glib::usleep(1000*50);
		
		if (!oDlg.update(   ((double)(xi+1))/(double)vecMod.size(),
							Util::compose(	_("Loading Cipher %s..."), 
											vecMod[xi].c_str())))
			break;
		
		if (Util::loadKernelModule(vecMod[xi]))
			ciLoaded++;
	}
	
	if (ciLoaded == 0)
	{
		MsgDialog::tConfig tConfigItem;
		MsgDialog::initConfig(tConfigItem);
		
		tConfigItem.bAutoRun = true;
		tConfigItem.bWaitData = true;
		tConfigItem.stTitle = _("Load File Systems");
		tConfigItem.stMsg = _("Failed loading modules.");
		
		MsgDialog oDlg(tConfigItem);
		
		return;
	}
}

//==============================================================================

void eMountWin::onLoadFSys()
{
	int ciLoaded = 0;
	std::vector<Glib::ustring> vecMod;
	
	if (!Pline::MountUtil::getAvailFSysMod(&vecMod) || vecMod.size() == 0)
	{
		MsgDialog::tConfig tConfigItem;
		MsgDialog::initConfig(tConfigItem);
		
		tConfigItem.bAutoRun = true;
		tConfigItem.bWaitData = true;
		tConfigItem.stTitle = _("Load File Systems");
		tConfigItem.stMsg = _("No modules found.");
		
		MsgDialog oDlg(tConfigItem);
		
		return;
	}
	
	// confirm operation
	MsgDialog::tConfig tConfigItemConfirm;
	MsgDialog::initConfig(tConfigItemConfirm);
		
	tConfigItemConfirm.bAutoRun = false;
	tConfigItemConfirm.bWaitData = true;
	tConfigItemConfirm.bYesNo = true;
	tConfigItemConfirm.stTitle = _("Load File Systems");
	tConfigItemConfirm.stMsg = _("Filesystems modules will be loaded into the Linux Kernel, confirm?");
		
	MsgDialog oConfirmDlg(tConfigItemConfirm);
	
	if (oConfirmDlg.run() != Gtk::RESPONSE_OK)
		return;
	
	// load
	ProgressDialog::tConfig tConfigItem;
	ProgressDialog::initConfig(tConfigItem);
	
	tConfigItem.bAutoShow = true;
	tConfigItem.stTitle = _("Load File Systems");
	tConfigItem.stMsg = _("Loading File Systems...");
	
	ProgressDialog oDlg(tConfigItem);
	
	oDlg.set_modal(true);
	
	for (   std::vector<Glib::ustring>::size_type xi = 0;
			xi<vecMod.size();
			++xi)
	{	
		Glib::usleep(1000*50);
		
		if (!oDlg.update(   ((double)(xi+1))/(double)vecMod.size(),
							Util::compose(	_("Loading Filesystem %s..."), 
											vecMod[xi].c_str())))
			break;
		
		if (Util::loadKernelModule(vecMod[xi]))
			ciLoaded++;
	}
	
	if (ciLoaded == 0)
	{
		MsgDialog::tConfig tConfigItem;
		MsgDialog::initConfig(tConfigItem);
		
		tConfigItem.bAutoRun = true;
		tConfigItem.bWaitData = true;
		tConfigItem.stTitle = _("Load File Systems");
		tConfigItem.stMsg = _("Failed loading modules.");
		
		MsgDialog oDlg(tConfigItem);
		
		return;
	}
}

//==============================================================================

void eMountWin::onMount()
{
	mount();
}

//==============================================================================

void eMountWin::onQuit()
{
	quit(true);
}

//==============================================================================

void eMountWin::onSelVolumeClone()
{
	if (!gpoVolumeTreeView)
		return;
	
	VolumeGuiState *poVolumeGuiState = gpoVolumeTreeView->getSelectedVolume();
	
	if (poVolumeGuiState)
		onVolumeClone(poVolumeGuiState);
}

//==============================================================================

void eMountWin::onSelVolumeEject()
{
	if (!gpoVolumeTreeView)
		return;
	
	VolumeGuiState *poVolumeGuiState = gpoVolumeTreeView->getSelectedVolume();
	
	if (poVolumeGuiState)
		onVolumeEject(poVolumeGuiState);
}

//==============================================================================

void eMountWin::onSelVolumeEnlarge()
{
	if (!gpoVolumeTreeView)
		return;
	
	VolumeGuiState *poVolumeGuiState = gpoVolumeTreeView->getSelectedVolume();
	
	if (poVolumeGuiState)
		onVolumeEnlarge(poVolumeGuiState);
}

//==============================================================================

void eMountWin::onSelVolumeInit()
{
	if (!gpoVolumeTreeView)
		return;
	
	VolumeGuiState *poVolumeGuiState = gpoVolumeTreeView->getSelectedVolume();
	
	if (poVolumeGuiState)
		onVolumeInit(poVolumeGuiState);
}

//==============================================================================

void eMountWin::onSelVolumeMount()
{	
	if (!gpoVolumeTreeView)
		return;
	
	VolumeGuiState *poVolumeGuiState = gpoVolumeTreeView->getSelectedVolume();
	
	if (poVolumeGuiState)
		onVolumeMount(poVolumeGuiState);
}

//==============================================================================

void eMountWin::onSelVolumeProp()
{
	PropDialog *poPropDlg = 0;
	std::vector<Glib::ustring> vecVolumeConfigLabel;
	
	if (!gpoVolumeTreeView)
		return;
		
	eMountSys::getInstance().getGuiBuilder()->get_widget_derived("PropDialog", poPropDlg);
	
	if (!poPropDlg)
		return;
	
	addOpenedDialog(poPropDlg);
	
	VolumeMountGui *poVolumeGui = (VolumeMountGui*)findVolume(gpoVolumeTreeView->getSelectedVolume());
	VolumeConfig *poVolumeConfigInit = findVolumeConfig(gpoVolumeTreeView->getSelectedVolume());

	if (!poVolumeGui || !poVolumeConfigInit)
		return;

	if (!poVolumeGui->lockThread())
	{
		VolumeDialog::tConfig tDlgConfig;	
		VolumeDialog::initConfig(tDlgConfig);
			
		tDlgConfig.bAutoRun = true;
		tDlgConfig.stLabel = poVolumeGui->getVolumeConfig().getData().getLabel();
		tDlgConfig.stMsg = _("Volume busy.");
		
		VolumeDialog oDlg(tDlgConfig);
			
		return;
	}
		
	VolumeConfig *poVolumeConfig = new VolumeConfig;
	*poVolumeConfig = *poVolumeConfigInit;

	getVolumeConfigLabelList(&vecVolumeConfigLabel);

	poPropDlg->setData( poVolumeConfig,
						&vecVolumeConfigLabel,
						&poVolumeGui->getPlineStatus(),
						poVolumeGui->getFSys(),
						poVolumeGui->getCipher(),
						poVolumeGui->getDiskSize(),
						poVolumeGui->getDiskAvailableSpace(),
						poVolumeGui->getDiskUsedSpace());
	
	gmtxUserIdle.lock();
	
	if (poPropDlg->run() == Gtk::RESPONSE_OK
		&& *poVolumeConfig != *poVolumeConfigInit)
	{
		updateVolumeConfig(poVolumeConfigInit, poVolumeConfig);
	}

	delete poVolumeConfig;
	
	poVolumeGui->lockThread(false);
	
	gmtxUserIdle.unlock();

	processQueueData();
}

//==============================================================================

void eMountWin::onSelVolumeRemove()
{
	VolumeConfig *poVolumeConfig = 0;
	VolumeMountGui *poVolumeGui = 0;
	VolumeGuiState *poVolumeGuiState = 0;
	
	if (!gpoVolumeTreeView)
		return;
	
	if (!(poVolumeGuiState = gpoVolumeTreeView->getSelectedVolume())
		|| !(poVolumeConfig = findVolumeConfig(poVolumeGuiState))
		|| !(poVolumeGui = (VolumeMountGui*)findVolume(poVolumeGuiState)))
	{
		return;
	}
	
	VolumeDialog::tConfig tDlgConfig;	
	VolumeDialog::initConfig(tDlgConfig);
			
	tDlgConfig.bAutoRun = true;
	tDlgConfig.stLabel = poVolumeGui->getVolumeConfig().getData().getLabel();
		
	if (!poVolumeGui->lockThread())
	{
		tDlgConfig.stMsg = _("Volume busy.");
		
		VolumeDialog oDlg(tDlgConfig);
			
		return;
	}
			
	if (poVolumeGui->isInUse())
	{
		tDlgConfig.stMsg = _("Volume in use.");
		
		VolumeDialog oDlg(tDlgConfig);
		
		poVolumeGui->lockThread(false);
		
		return;
	}
	
	VolumeDialog::tConfig tDlgConfigConfirm;
	VolumeDialog::initConfig(tDlgConfigConfirm);
			
	tDlgConfigConfirm.bAutoRun = false;
	tDlgConfigConfirm.bCancelButton = true;
	tDlgConfigConfirm.bCancelButtonHasFocus = true;
	tDlgConfigConfirm.bYesNo = true;
	tDlgConfigConfirm.stLabel = poVolumeGui->getVolumeConfig().getData().getLabel();
	tDlgConfigConfirm.stMsg = _("Remove volume from list?");

	VolumeDialog oDlg(tDlgConfigConfirm); 
	
	gmtxUserIdle.lock();
	
	if (oDlg.run() != Gtk::RESPONSE_OK)
	{
		poVolumeGui->lockThread(false);
	
		gmtxUserIdle.unlock();
		
		return;
	}
	
	gmtxUserIdle.unlock();
	
	gpoVolumeTreeView->removeVolume(poVolumeGuiState);
	QConfig::getInstance().removeVolume(poVolumeConfig);
	
	Glib::Mutex::Lock lock(gmtxVolumeGuiLst);
	
	glstVolumeGui.remove(poVolumeGui);
	delete poVolumeGui;
	
	std::list<VolumeConfig*> lstVolumeConfig = QConfig::getInstance().getVolumes();
	lstVolumeConfig.sort(VolumeConfig::cmpByPos);
	
	updateVolumeConfigPositions(lstVolumeConfig);
}

//==============================================================================

void eMountWin::onSortBy(const int i)
{
	if (!((Gtk::RadioMenuItem&)goViewMenu.items()[kMenuViewSortByLabel]).get_active()
		&& i == kMenuViewSortByPos)
	{
		// Sort by Position
		QConfig::getInstance().setSortBy((int)kSortByPos);
		
		if (gpoVolumeTreeView)
			gpoVolumeTreeView->sortByPos();
		
	}
	else if (!((Gtk::RadioMenuItem&)goViewMenu.items()[kMenuViewSortByPos]).get_active()
			&& i == kMenuViewSortByLabel)
	{
		// Sort by Label
		QConfig::getInstance().setSortBy((int)kSortByLabel);
		
		if (gpoVolumeTreeView)
			gpoVolumeTreeView->sortByLabel();
	}
}

//==============================================================================

void eMountWin::onVolumeCancel(VolumeGuiState* poVolumeGuiState)
{
	VolumeMountGui *poVolumeGui = 0;
	
	if ((poVolumeGui = (VolumeMountGui*)findVolume(poVolumeGuiState)))
	{
		VolumeDialog::tConfig tDlgConfigConfirm;
		VolumeDialog::initConfig(tDlgConfigConfirm);
				
		tDlgConfigConfirm.bAutoRun = false;
		tDlgConfigConfirm.bCancelButton = true;
		tDlgConfigConfirm.bCancelButtonHasFocus = true;
		tDlgConfigConfirm.bYesNo = true;
		tDlgConfigConfirm.stLabel = poVolumeGui->getVolumeConfig().getData().getLabel();
		tDlgConfigConfirm.stMsg = _("Confirm Abort?");
		
		VolumeDialog oDlg(tDlgConfigConfirm); 
		
		gmtxUserIdle.lock();
		
		if (oDlg.run() != Gtk::RESPONSE_OK)
		{
			gmtxUserIdle.unlock();
			return;
		}
		
		gmtxUserIdle.unlock();
		
		poVolumeGui->cancel();
	
		if (gpoVolumeTreeView)
			gpoVolumeTreeView->updateVolume(poVolumeGuiState);
	}
}

//==============================================================================

void eMountWin::onVolumeClone(VolumeGuiState* poVolumeGuiState)
{
	VolumeMountGui *poVolumeGui = 0;
	
	if ((poVolumeGui = (VolumeMountGui*)findVolume(poVolumeGuiState)))
		poVolumeGui->clone();
	
	if (gpoVolumeTreeView)
		gpoVolumeTreeView->updateVolume(poVolumeGuiState);
}

//==============================================================================

void eMountWin::onVolumeDClick(VolumeGuiState* poVolumeGuiState)
{
	bool bUpdate = false;
	
	if (poVolumeGuiState->isMountMode() || poVolumeGuiState->isEjectMode())
	{
		VolumeMountGui *poVolumeGui = 0;
	
		if ((poVolumeGui = (VolumeMountGui*)findVolume(poVolumeGuiState)))
		{			
			(poVolumeGuiState->isMountMode()) ?
				poVolumeGui->mount() : poVolumeGui->eject();
				
			bUpdate = true;
		}
	}
	else if (!poVolumeGuiState->isProgressMode())
	{
		poVolumeGuiState->setMountMode();
		
		bUpdate = true;
	}

	if (gpoVolumeTreeView && bUpdate)
		gpoVolumeTreeView->updateVolume(poVolumeGuiState);
}

//==============================================================================

void eMountWin::onVolumeEject(VolumeGuiState* poVolumeGuiState)
{
	VolumeMountGui *poVolumeGui = 0;
	
	if ((poVolumeGui = (VolumeMountGui*)findVolume(poVolumeGuiState)))
		poVolumeGui->eject();
	
	if (gpoVolumeTreeView)
		gpoVolumeTreeView->updateVolume(poVolumeGuiState);
}

//==============================================================================

void eMountWin::onVolumeEnlarge(VolumeGuiState* poVolumeGuiState)
{
	VolumeMountGui *poVolumeGui = 0;
	EnlargeVolumeDialog *poEnlargeDlg = 0;
	
	if (!(poVolumeGui = (VolumeMountGui*)findVolume(poVolumeGuiState)))
		return;
		
	VolumeConfig &oVolumeConfig = poVolumeGui->getVolumeConfig();
	
	eMountSys::getInstance().getGuiBuilder()->get_widget_derived("EnlargeVolumeDialog", poEnlargeDlg);
	
	if (!poEnlargeDlg)
		return;
	
	addOpenedDialog(poEnlargeDlg);
	
	unsigned long long ullNewVolumeSize = 0;
	unsigned long long ullCurrVolumeSize = 0;
	
	VolumeDialog::tConfig tDlgConfig;	
	VolumeDialog::initConfig(tDlgConfig);

	tDlgConfig.stLabel = oVolumeConfig.getData().getLabel();
		
	if (!Util::isRegularFile(oVolumeConfig.getData().getSrc().c_str()))
	{	
		tDlgConfig.stMsg = _("Enlarge can be used only for disk images.");
		
		VolumeDialog oDlg(tDlgConfig);
		
		return;
	}
	
	if (!Util::getFileSize(	oVolumeConfig.getData().getSrc().c_str(),
							&ullCurrVolumeSize))
	{
		tDlgConfig.stMsg = _("File size unknown.");
		
		VolumeDialog oDlg(tDlgConfig);
		
		return;
	}
	
	if (!oVolumeConfig.getMountData())
	{
		tDlgConfig.stMsg = _("Mount data unknown.");
		
		VolumeDialog oDlg(tDlgConfig);
		
		return;
	}
	
	// set fys
	Glib::ustring stFSys = (poVolumeGui->getFSys().empty()) ?
		oVolumeConfig.getMountData()->getFSys()
			: poVolumeGui->getFSys();
			
	// check fsys recognized
	if (stFSys.empty())
	{
		tDlgConfig.stMsg = _("Volume must be mounted.");
		
		VolumeDialog oDlg(tDlgConfig);
		
		return;
	}
	
	// check can resize fsys
	std::vector<Glib::ustring> vecAllowedFSys;
	Pline::FSysUtil::getEnlargeFSysList(&vecAllowedFSys);
	
	if (!Util::findSt(stFSys, vecAllowedFSys))
	{	
		tDlgConfig.stMsg = Util::compose(_("Filesystem %s not supported."), stFSys.c_str());
		
		VolumeDialog oDlg(tDlgConfig);
		
		return;
	}
	
	// check key file is present
	if (oVolumeConfig.getCryptData())
	{
		if (oVolumeConfig.getCryptData()->getIsEncrypted()
			&& !oVolumeConfig.getCryptData()->getKeyFile().empty()
			&& !Util::isRegularFile(oVolumeConfig.getCryptData()->getKeyFile().c_str()))
		{
			tDlgConfig.stMsg = _("Encryption key file not found.");
		
			VolumeDialog oDlg(tDlgConfig);
			
			return;
		}
	}
	
	// enlarge dialog
	poEnlargeDlg->setData(	&poVolumeGui->getVolumeConfig(),
							&ullNewVolumeSize,
							ullCurrVolumeSize);
	
	gmtxUserIdle.lock();
	
	if (poEnlargeDlg->run() == Gtk::RESPONSE_OK)
		poVolumeGui->enlargeVolume(ullNewVolumeSize-ullCurrVolumeSize);
		
	gmtxUserIdle.unlock();
	
	processQueueData();
}

//==============================================================================

void eMountWin::onVolumeEnterPressed(VolumeGuiState* poVolumeGuiState)
{
	VolumeMountGui *poVolumeGui = 0;
	bool bMounted = false;
	
	if (!(poVolumeGui = (VolumeMountGui*)findVolume(poVolumeGuiState)))
		return;
	
	if (poVolumeGui->isInUse(&bMounted))
	{
		if (bMounted)
			poVolumeGui->eject();
	}
	else
		poVolumeGui->mount();
		
	gpoVolumeTreeView->updateVolume(poVolumeGuiState);
}

//==============================================================================

void eMountWin::onVolumeInit(VolumeGuiState* poVolumeGuiState)
{
	VolumeMountGui *poVolumeGui = 0;
	InitVolumeDialog *poInitDlg = 0;
	VolumeConfig *poNewVolumeConfig = 0;
	
	if (!(poVolumeGui = (VolumeMountGui*)findVolume(poVolumeGuiState)))
		return;
		
	eMountSys::getInstance().getGuiBuilder()->get_widget_derived("InitVolumeDialog", poInitDlg);
	
	if (!poInitDlg)
		return;
	
	addOpenedDialog(poInitDlg);
	
	Glib::ustring stCipher;
	Glib::ustring stCBC;
	Glib::ustring stKeySize;
	
	if (!poInitDlg->askConfirm(&poVolumeGui->getVolumeConfig()))
		return;
		
	poNewVolumeConfig = new VolumeConfig;
	*poNewVolumeConfig = poVolumeGui->getVolumeConfig();
	
	poInitDlg->setData( poNewVolumeConfig,
						&stCipher,
						&stCBC,
						&stKeySize);
	
	gmtxUserIdle.lock();
	
	if (poInitDlg->run() == Gtk::RESPONSE_OK)
	{
		poVolumeGui->initVolume(	poNewVolumeConfig,
									stCipher,
									stCBC,
									stKeySize);
	}
	
	delete poNewVolumeConfig;
		
	gmtxUserIdle.unlock();
	
	processQueueData();
}

//==============================================================================

void eMountWin::onVolumeMount(VolumeGuiState* poVolumeGuiState)
{	
	VolumeMountGui *poVolumeGui = 0;
	
	if ((poVolumeGui = (VolumeMountGui*)findVolume(poVolumeGuiState)))
		poVolumeGui->mount();
	
	if (gpoVolumeTreeView)
		gpoVolumeTreeView->updateVolume(poVolumeGuiState);
}

//==============================================================================

void eMountWin::onVolumePopupMenu(  VolumeGuiState* poVolumeGuiState, 
									GdkEventButton* event)
{
	if (!event)
		return;
		
	goVolumeMenu.popup(event->button, event->time);
}
									
//==============================================================================

void eMountWin::onVolumeSelectionChanged(VolumeGuiState* poVolumeGuiState)
{
	enableVolumeMenu((poVolumeGuiState) ? true : false);
}

//==============================================================================
 
bool eMountWin::onWinDelete(GdkEventAny* event)
{
	return (quit()) ? false : true;
}
 
//==============================================================================
 
void eMountWin::onWinResize(GdkEventConfigure* event)
{
	int ix, iy;
	
	// save position in member var
	get_position(ix, iy);
	
	giWinPosX = ix;
	giWinPosY = iy;
	
	// save size in member var
	get_size(ix, iy);
	
	giWinSizeX = ix;
	giWinSizeY = iy;
}

//==============================================================================

void eMountWin::processQueueData()
{	
	eMountWin *poEMountWin = eMountWin::getInstance();
	
	if (!poEMountWin)
		return;

	std::vector<DispatchData*> vecToProcess;
	std::vector<DispatchData*> vecProcessed;
	
	poEMountWin->gmtxDispatchData.lock();
	
	for (	std::list<DispatchData*>::iterator ite = poEMountWin->glstDispatchData.begin();
			ite != poEMountWin->glstDispatchData.end();
			++ite)
	{
		DispatchData *poDispatchData = (DispatchData*)*ite;
		
		// process
		if (!poDispatchData->isReqUserData())
		{
			poDispatchData->process();
			
			vecProcessed.push_back(poDispatchData);
		}
		else
			vecToProcess.push_back(poDispatchData);
	}
	
	poEMountWin->gmtxDispatchData.unlock();
	
	// process blocking requests
	for (	std::vector<DispatchData*>::size_type xi = 0;
			xi < vecToProcess.size();
			++xi)
	{
		DispatchData *poDispatchData = vecToProcess[xi];
		
		if (!poEMountWin->gmtxUserIdle.trylock())
			continue;
			
		if (!poEMountWin->gbUserIdle)
		{
			poEMountWin->gmtxUserIdle.unlock();
			continue;
		}
			
		poEMountWin->gbUserIdle = false;
		
		poDispatchData->process();
		
		vecProcessed.push_back(poDispatchData);
			
		poEMountWin->gbUserIdle = true;
			
		poEMountWin->gmtxUserIdle.unlock();
	}
	
	// remove processed elements
	for (	std::vector<DispatchData*>::size_type xi = 0;
			xi < vecProcessed.size();
			++xi)
	{
		poEMountWin->gmtxDispatchData.lock();
		poEMountWin->glstDispatchData.remove(vecProcessed[xi]);
		poEMountWin->gmtxDispatchData.unlock();
		
		delete vecProcessed[xi];
	}
	
	if (vecProcessed.size() > 0) // some processed could call this method and got it busy
		processQueueData(); 	
}

//==============================================================================

void eMountWin::queueData(DispatchData *poDispatchData)
{
	eMountWin *poEMountWin = eMountWin::getInstance();
	
	if (poEMountWin)
	{
		Glib::Mutex::Lock lock(poEMountWin->gmtxDispatchData);
		
		poEMountWin->glstDispatchData.push_back(poDispatchData);
	}
}

//==============================================================================

bool eMountWin::quit(const bool bQuitLoop)
{	
	bool bErr = false;
	std::vector<VolumeGui*> vecLocked;
	
	VolumeDialog::tConfig tDlgConfig;	
	VolumeDialog::initConfig(tDlgConfig);

	if (gpoAutomount)
		gpoAutomount->stop();

	gmtxVolumeGuiLst.lock();
		
	// lock threads
	for (   std::list<VolumeGui*>::iterator ite = glstVolumeGui.begin();
			ite != glstVolumeGui.end();
			++ite)
	{
		VolumeMountGui* poVolumeGui = (VolumeMountGui*)*ite;
			
		tDlgConfig.stLabel = poVolumeGui->getVolumeConfig().getData().getLabel();
		
		if (poVolumeGui->lockThread())
			vecLocked.push_back(poVolumeGui);
		else
		{
			tDlgConfig.stMsg = _("Volume busy.");
					
			bErr = true;
			break;
		}
		
		bool bMounted = false;
		
		if (poVolumeGui->isInUse(&bMounted) && !bMounted)
		{
			tDlgConfig.stMsg = _("Volume in use.");
		
			bErr = true;
			break;
		}
	}

	if (bErr)
	{
		// unlock volumes
		for (   std::vector<VolumeGui*>::size_type xi = 0;
				xi < vecLocked.size();
				++xi)
		{
			vecLocked[xi]->lockThread(false);
		}

		tDlgConfig.bAutoRun = true;
				
		VolumeDialog oDlg(tDlgConfig);

		// ask force quit
		MsgDialog::tConfig tConfigItem;
		MsgDialog::initConfig(tConfigItem);
		
		tConfigItem.bAutoRun = false;
		tConfigItem.bYesNo = true;
		tConfigItem.stTitle = _("Quit");
		tConfigItem.stMsg = _("Force quit?");
		
		MsgDialog oMsgDlg(tConfigItem);
		
		if (oMsgDlg.run() != Gtk::RESPONSE_OK)
		{
			gmtxVolumeGuiLst.unlock();
			
			return false;
		}
		else
		{
			// try to lock threads
			for (   std::list<VolumeGui*>::iterator ite = glstVolumeGui.begin();
					ite != glstVolumeGui.end();
					++ite)
			{
				VolumeMountGui* poVolumeGui = (VolumeMountGui*)*ite;
				poVolumeGui->lockThread();
			}
		}
	}
	
	if (gpoThreadVolumeStatus)
	{
		gpoThreadVolumeStatus->join();
		
		delete gpoThreadVolumeStatus;
		gpoThreadVolumeStatus = 0;
	}

	gmtxVolumeGuiLst.unlock();

	if (bQuitLoop)
		Gtk::Main::quit();
				
	return true;
}

//==============================================================================

void eMountWin::setChild(Gtk::Window &window)
{
	eMountWin *poEMountWin = eMountWin::getInstance();
	
	if (poEMountWin)
		window.set_transient_for(*poEMountWin);
}

//==============================================================================

void eMountWin::setUserIdle(const bool bIdle)
{
	eMountWin *poEMountWin = eMountWin::getInstance();
	
	if (poEMountWin)
	{
		poEMountWin->gmtxUserIdle.lock();

		poEMountWin->gbUserIdle = bIdle;
		
		poEMountWin->gmtxUserIdle.unlock();
		
		if (bIdle)
			eMountWin::processQueueData();
	}
}

//==============================================================================

void eMountWin::updateVolumeConfig( VolumeConfig *poOldVolumeConfig,
									VolumeConfig *poNewVolumeConfig)
{
	// update position
	if (poOldVolumeConfig->getData().getPos() != poNewVolumeConfig->getData().getPos())
	{
		Glib::Mutex::Lock lock(gmtxVolumeGuiLst);
	
		std::list<VolumeConfig*> lstVolumeConfig = QConfig::getInstance().getVolumes();
		std::list<VolumeConfig*>::iterator ite_old;
		std::list<VolumeConfig*>::iterator ite_new;

		lstVolumeConfig.sort(VolumeConfig::cmpByPos);
	
		for (   ite_old = lstVolumeConfig.begin();
				ite_old != lstVolumeConfig.end();
				++ite_old)
		{
			if (*ite_old == poOldVolumeConfig)
				break;
		}
	
		assert(ite_old != lstVolumeConfig.end());
	
		for (   ite_new = lstVolumeConfig.begin();
				ite_new != lstVolumeConfig.end();
				++ite_new)
		{
			if (((VolumeConfig*)*ite_new)->getData().getPos() == poNewVolumeConfig->getData().getPos())
			{
				if (poOldVolumeConfig->getData().getPos() < poNewVolumeConfig->getData().getPos())
					++ite_new;
					
				break;
			}
		}
			
		// splice	
		lstVolumeConfig.splice(ite_new, lstVolumeConfig, ite_old);
		
		updateVolumeConfigPositions(lstVolumeConfig);
	}
	
	// update label
	if (poOldVolumeConfig->getData().getLabel() != poNewVolumeConfig->getData().getLabel())
	{
		// update volumes gui and data
		for (   std::list<VolumeGui*>::iterator ite_gui = glstVolumeGui.begin();
				ite_gui != glstVolumeGui.end();
				++ite_gui)
		{
			VolumeGui *poVolumeGui = (VolumeGui*)*ite_gui;
			
			if (&poVolumeGui->getVolumeConfig() == poOldVolumeConfig)
			{	
				VolumeGuiState *poVolumeGuiState = poVolumeGui->getVolumeGuiState();
				
				poVolumeGuiState->setLabel(poNewVolumeConfig->getData().getLabel());
				
				if (gpoVolumeTreeView)
					gpoVolumeTreeView->updateVolume(poVolumeGuiState);
			}
		}
	}
	
	*poOldVolumeConfig = *poNewVolumeConfig;
}

//==============================================================================
		
void eMountWin::updateVolumeConfigPositions(std::list<VolumeConfig*>& lstVolumeConfig)
{		
	int ci = 0;
	
	for (   std::list<VolumeConfig*>::iterator ite = lstVolumeConfig.begin();
			ite != lstVolumeConfig.end();
			++ite)
	{
		((VolumeConfig*)*ite)->getData().setPos(ci+1);
			
		++ci;
	}
		
	// update volumes gui
	for (   std::list<VolumeGui*>::iterator ite_gui = glstVolumeGui.begin();
			ite_gui != glstVolumeGui.end();
			++ite_gui)
	{
		VolumeGui *poVolumeGui = (VolumeGui*)*ite_gui;
			
		for (   std::list<VolumeConfig*>::iterator ite_conf = lstVolumeConfig.begin();
				ite_conf != lstVolumeConfig.end();
				++ite_conf)
		{
			VolumeConfig *poVolumeConfig = (VolumeConfig*)*ite_conf;
				
			if (&poVolumeGui->getVolumeConfig() == poVolumeConfig)
			{
				VolumeGuiState *poVolumeGuiState = poVolumeGui->getVolumeGuiState();
					
				poVolumeGuiState->setPos(poVolumeConfig->getData().getPos());
					
				if (gpoVolumeTreeView)
					gpoVolumeTreeView->updateVolume(poVolumeGuiState);
			}
		}
	}
	
	// sort volumetreeview (2.12.7 needs this)
	if (gpoVolumeTreeView)
	{
		if (((Gtk::RadioMenuItem&)goViewMenu.items()[kMenuViewSortByPos]).get_active())
			gpoVolumeTreeView->sortByPos();
		else if (((Gtk::RadioMenuItem&)goViewMenu.items()[kMenuViewSortByLabel]).get_active())
			gpoVolumeTreeView->sortByLabel();
	}
	
}

//==============================================================================

void eMountWin::updateVolumeGuiState(VolumeGuiState* poVolumeGuiState)
{
	eMountWin *poEMountWin = eMountWin::getInstance();
	
	if (poEMountWin)
	{
		if (poEMountWin->gpoVolumeTreeView)
			poEMountWin->gpoVolumeTreeView->updateVolume(poVolumeGuiState);
	}
}

//==============================================================================
