/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef EMOUNTWIN_H
#define EMOUNTWIN_H

#include <gtkmm.h>
#include <vector>
#include <queue>
#include <list>

#include "QDialog.h"

#include "VolumeTreeView.h"
#include "VolumeGui.h"

class Automount;
class DispatchData;
class VolumeGuiState;
class ThreadVolumeStatus;
class VolumeMountGui;

class eMountWin : public Gtk::Window
{
public:
	eMountWin(  BaseObjectType* cobject,
				const Glib::RefPtr<Gtk::Builder>& refGlade);
	virtual ~eMountWin();

	// set children
	static void setChild(Gtk::Window &window);
	static Gtk::Window* getWindow() {return eMountWin::getInstance();}
	
	// dialogs to delete
	static void addOpenedDialog(Gtk::Dialog *po);
	
	// data request
	static bool hasDispatchData(void *p, int *piCount = 0);
	static void processQueueData();
	static void queueData(DispatchData *poDispatchData);
	
	static void setUserIdle(const bool bIdle);
	
	// VolumeTreeView
	static void updateVolumeGuiState(VolumeGuiState* poVolumeGuiState);
	
private:
	Glib::RefPtr<Gtk::Builder> gpoRefGlade;
	
	virtual void onVolumeCancel(VolumeGuiState* poVolumeGuiState);
	virtual void onVolumeClone(VolumeGuiState* poVolumeGuiState);
	virtual void onVolumeDClick(VolumeGuiState* poVolumeGuiState);
	virtual void onVolumeEject(VolumeGuiState* poVolumeGuiState);
	virtual void onVolumeEnlarge(VolumeGuiState* poVolumeGuiState);
	virtual void onVolumeEnterPressed(VolumeGuiState* poVolumeGuiState);
	virtual void onVolumeInit(VolumeGuiState* poVolumeGuiState);
	virtual void onVolumeMount(VolumeGuiState* poVolumeGuiState);
	virtual void onVolumePopupMenu( VolumeGuiState* poVolumeGuiState, 
									GdkEventButton* event);
	virtual void onVolumeSelectionChanged(VolumeGuiState* poVolumeGuiState);
	
	virtual void on_drop_drag_data_received(const Glib::RefPtr<Gdk::DragContext>& context, 
											int, 
											int,
											const Gtk::SelectionData& selection_data, 
											guint, 
											guint time);
	
private:
	static eMountWin* gs_window;
		
	static eMountWin* getInstance();
	bool quit(const bool bQuitLoop = false);
	
	// Dialogs
	std::vector<Gtk::Dialog*> gvecOpenedDialog;
	
	// Ask data to user
	Glib::Mutex			gmtxUserIdle;
	bool				gbUserIdle;
	
	Glib::Mutex 				gmtxDispatchData;
	std::list<DispatchData*> 	glstDispatchData;
	
	// Volumes
	Glib::Mutex				gmtxVolumeGuiLst;
	std::list<VolumeGui*>   glstVolumeGui;
	
	VolumeMountGui* addVolume(VolumeConfig* poVolumeConfig);
	
	VolumeGui* findVolume(VolumeGuiState* poVolumeGuiState);
	VolumeConfig* findVolumeConfig(VolumeGuiState* poVolumeGuiState);
	void getVolumeConfigLabelList(std::vector<Glib::ustring> *pvecVolumeConfigLabel);
	
	void mount(const Glib::ustring& stSrc = "");
	
	void updateVolumeConfig(VolumeConfig *poOldVolumeConfig,
							VolumeConfig *poNewVolumeConfig);
	void updateVolumeConfigPositions(std::list<VolumeConfig*>& lstVolumeConfig);
	
	// Volume status thread;
	ThreadVolumeStatus *gpoThreadVolumeStatus;
	
	void launchVolumeStatusThread();
	
	// Automount
	Automount *gpoAutomount;
	
	void launchAutomount();
	
	// Window pos/size
	int giWinPosX, giWinPosY, giWinSizeX, giWinSizeY;

	// Internal callbacks
	void onAbout();
	void onCreateDiskImage();
	void onCreatePlainDiskImage();
	void onCreateVolume();
	void onEjectAll();
	void onLoadCipher();
	void onLoadFSys();
	void onMount();
	void onSortBy(const int i);
	void onQuit();
	void onSelVolumeClone();
	void onSelVolumeEject();
	void onSelVolumeEnlarge();
	void onSelVolumeInit();
	void onSelVolumeMount();
	void onSelVolumeProp();
	void onSelVolumeRemove();
	bool onWinDelete(GdkEventAny* event);
	void onWinResize(GdkEventConfigure* event);
	
	// MenuBar
	enum
	{
		kMenuFileNew = 0,
		kMenuFileMount = 2,
		kMenuFileLoadCipher = 4,
		kMenuFileLoadFSys = 5,
		kMenuFileEjectAll = 7,
		kMenuFileDisableAutomount = 8,
		kMenuFileQuit = 10,
	};
	
	enum
	{
		kMenuViewSortByLabel = 0,
		kMenuViewSortByPos = 1,
		kMenuViewSaveWindowPos = 3,
		kMenuViewSaveWindowSize = 4,
	};
	
	Gtk::MenuBar	goMenuBar;
	Gtk::Menu		goFileMenu;
	Gtk::Menu		goFileNewMenu;
	Gtk::Menu		goVolumeMenu;
	Gtk::Menu		goViewMenu;
	Gtk::Menu		goHelpMenu;
	
	void enableVolumeMenu(const bool bEnable);
	void buildMenuBar();
	
	// Volumes view	
	VolumeTreeView *gpoVolumeTreeView;
	
	// Sort
	enum {
		kSortByPos = 0,
		kSortByLabel = 1,
	};
};
	
#endif /* EMOUNTWIN_H */
